@@ -16,12 +16,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.78 2004/11/01 14:39:24 chpe Exp $
+ *  $Id: downloader-view.c,v 1.79 2004/11/02 20:56:55 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h" 
-#endif
 
 #include "downloader-view.h"
 #include "ephy-file-helpers.h"
