@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-persist.h,v 1.14 2004/03/27 21:51:59 adamh Exp $
+ *  $Id: ephy-embed-persist.h,v 1.15 2004/09/18 07:50:09 marco Exp $
  */
 
 #ifndef EPHY_EMBED_PERSIST_H
@@ -47,7 +47,8 @@ typedef enum
 	EMBED_PERSIST_COPY_PAGE		= 1 << 0,
 	EMBED_PERSIST_MAINDOC		= 1 << 1,
 	EMBED_PERSIST_NO_VIEW		= 1 << 2,
-	EMBED_PERSIST_ASK_DESTINATION	= 1 << 3
+	EMBED_PERSIST_ASK_DESTINATION	= 1 << 3,
+	EMBED_PERSIST_DO_CONVERSION     = 1 << 4
 } EmbedPersistFlags;
 
 struct _EphyEmbedPersist
