@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,64 +14,65 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-embed-shell.h,v 1.14 2003/12/29 21:05:05 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_SHELL_H
 #define EPHY_EMBED_SHELL_H
 
-#include "ephy-embed.h"
-#include "ephy-embed-single.h"
-#include "ephy-history.h"
-
 #include <glib-object.h>
 #include <glib.h>
 
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_EMBED_SHELL		(ephy_embed_shell_get_type ())
-#define EPHY_EMBED_SHELL_IMPL		(ephy_embed_shell_get_impl ())
 #define EPHY_EMBED_SHELL(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_EMBED_SHELL, EphyEmbedShell))
 #define EPHY_EMBED_SHELL_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellClass))
 #define EPHY_IS_EMBED_SHELL(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_EMBED_SHELL))
 #define EPHY_IS_EMBED_SHELL_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED_SHELL))
 #define EPHY_EMBED_SHELL_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellClass))
 
-typedef struct EphyEmbedShellClass EphyEmbedShellClass;
-typedef struct EphyEmbedShell EphyEmbedShell;
-typedef struct EphyEmbedShellPrivate EphyEmbedShellPrivate;
+typedef struct EphyEmbedShellClass	EphyEmbedShellClass;
+typedef struct EphyEmbedShell		EphyEmbedShell;
+typedef struct EphyEmbedShellPrivate	EphyEmbedShellPrivate;
 
 extern EphyEmbedShell *embed_shell;
 
+typedef enum
+{
+	EPHY_MIME_PERMISSION_SAFE	= 1,
+	EPHY_MIME_PERMISSION_UNSAFE	= 2,
+	EPHY_MIME_PERMISSION_UNKNOWN	= 3
+} EphyMimePermission;
+
 struct EphyEmbedShell
 {
 	GObject parent;
-        EphyEmbedShellPrivate *priv;
+
+	/*< private >*/
+	EphyEmbedShellPrivate *priv;
 };
 
 struct EphyEmbedShellClass
 {
-        GObjectClass parent_class;
-
-	/* Methods */
-	EphyHistory * (* get_global_history)  (EphyEmbedShell *shell);
-	GObject     * (* get_downloader_view) (EphyEmbedShell *shell);
+	GObjectClass parent_class;
 };
 
-GType             ephy_embed_shell_get_type            (void);
+GType		   ephy_embed_shell_get_type		(void);
 
-GType             ephy_embed_shell_get_impl            (void);
+GObject		  *ephy_embed_shell_get_favicon_cache	(EphyEmbedShell *ges);
 
-EphyEmbedShell   *ephy_embed_shell_new                 (const char *type);
+GObject		  *ephy_embed_shell_get_global_history	(EphyEmbedShell *shell);
 
-GObject          *ephy_embed_shell_get_favicon_cache   (EphyEmbedShell *ges);
+GObject		  *ephy_embed_shell_get_downloader_view	(EphyEmbedShell *shell);
 
-EphyHistory      *ephy_embed_shell_get_global_history  (EphyEmbedShell *shell);
+GObject		  *ephy_embed_shell_get_encodings	(EphyEmbedShell *shell);
 
-GObject          *ephy_embed_shell_get_downloader_view (EphyEmbedShell *shell);
+GObject		  *ephy_embed_shell_get_embed_single	(EphyEmbedShell *shell);
 
-GObject		 *ephy_embed_shell_get_encodings       (EphyEmbedShell *shell);
-
-EphyEmbedSingle  *ephy_embed_shell_get_embed_single    (EphyEmbedShell *shell);
+EphyMimePermission ephy_embed_shell_check_mime		(EphyEmbedShell *shell,
+							 const char *mime_type);
 
 G_END_DECLS
 
