@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.h,v 1.41.4.2 2004/10/30 17:49:42 chpe Exp $
+ *  $Id: ephy-embed.h,v 1.44 2004/11/16 14:21:23 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_H
@@ -165,6 +165,7 @@ struct _EphyEmbedIface
 	void		   (* get_security_level)	(EphyEmbed *embed,
 						  	 EmbedSecurityLevel *level,
 						  	 char **description);
+	void		   (* show_page_certificate)	(EphyEmbed *embed);
 	void		   (* set_zoom)			(EphyEmbed *embed,
 							 float zoom);
 	float		   (* get_zoom)			(EphyEmbed *embed);
@@ -243,6 +244,8 @@ void		  ephy_embed_get_security_level		(EphyEmbed *embed,
 							 EmbedSecurityLevel *level,
 						 	 char **description);
 
+void		  ephy_embed_show_page_certificate	(EphyEmbed *embed);
+
 /* Zoom */
 void		  ephy_embed_set_zoom			(EphyEmbed *embed,
 							 float zoom);
