@@ -18,21 +18,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.cpp,v 1.60 2004/07/31 13:41:09 chpe Exp $
+ *  $Id: ContentHandler.cpp,v 1.68 2004/11/18 13:50:46 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
-#include <gtk/gtkimage.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtklabel.h>
 #include <gtk/gtkdialog.h>
+#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkstock.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
@@ -44,6 +39,10 @@
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsCExternalHandlerService.h>
 
+#ifdef ALLOW_PRIVATE_API
+#include <nsIServiceManager.h>
+#endif
+
 #include "ephy-prefs.h"
 #include "ephy-embed-single.h"
 #include "ephy-embed-shell.h"
@@ -54,14 +53,10 @@
 #include "eel-gconf-extensions.h"
 
 #include "ContentHandler.h"
-#include "MozillaPrivate.h"
 #include "MozDownload.h"
+#include "EphyUtils.h"
 
-class GContentHandler;
-
-NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
-
-#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
 GContentHandler::GContentHandler()
 {
 	LOG ("GContentHandler ctor (%p)", this)
@@ -77,14 +72,15 @@ GContentHandler::~GContentHandler()
 {
 	LOG ("GContentHandler dtor (%p)", this)
 
-#if !MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
-	nsMemory::Free (mMimeType);
+#ifndef MOZ_NSIMIMEINFO_NSACSTRING_
+	if (mMimeType)
+	{
+		nsMemory::Free (mMimeType);
+	}
 #endif
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// begin nsIHelperAppLauncher impl
-////////////////////////////////////////////////////////////////////////////////
+NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
@@ -101,7 +97,7 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 	NS_ENSURE_SUCCESS (rv, rv);
 
 	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
-#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
 	g_signal_emit_by_name (single, "handle_content", mMimeType.get(),
 			       mUrl.get(), &handled);
 #else
@@ -111,7 +107,7 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 
 	if (!handled)
 	{
-		MIMEDoAction ();
+		MIMEInitiateAction ();
 	}
 	else
 	{
@@ -143,7 +139,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 	}
 
 	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (aWindowContext);
-	GtkWidget *parentWindow = GTK_WIDGET (MozillaFindGtkParent (parentDOMWindow));
+	GtkWidget *parentWindow = GTK_WIDGET (EphyUtils::FindGtkParent (parentDOMWindow));
 
 	dialog = ephy_file_chooser_new (_("Save"), parentWindow,
 					GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -151,6 +147,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 					EPHY_FILE_FILTER_ALL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), defaultFile.get());
 
+	/* FIXME: modal -- mozilla sucks! */
 	do
 	{
 		g_free (filename);
@@ -182,56 +179,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 	}
 }
 
-NS_METHOD GContentHandler::LaunchHelperApp (void)
-{
-	nsCOMPtr<nsIExternalHelperAppService> helperService;
-
-	helperService = do_GetService (NS_EXTERNALHELPERAPPSERVICE_CONTRACTID);
-	NS_ENSURE_TRUE (helperService, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsPIExternalAppLauncher> appLauncher = do_QueryInterface (helperService);
-	NS_ENSURE_TRUE (appLauncher, NS_ERROR_FAILURE);
-	appLauncher->DeleteTemporaryFileOnExit(mTempFile);
-
-	GList *params = NULL;
-	char *param;
-	
-	param = gnome_vfs_make_uri_canonical (mUrl.get());
-	params = g_list_append (params, param);
-	gnome_vfs_mime_application_launch (mHelperApp, params);
-	g_free (param);
-	g_list_free (params);
-
-	mLauncher->Cancel();
-
-	return NS_OK;
-}
-
-NS_METHOD GContentHandler::CheckAppSupportScheme (void)
-{
-	GList *l;
-
-	mAppSupportScheme = PR_FALSE;	
-
-	if (!mHelperApp) return NS_OK;
-
-	if (mHelperApp->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
-		return NS_OK;
-	
-	for (l = mHelperApp->supported_uri_schemes; l != NULL; l = l->next)
-	{
-		char *uri_scheme = (char *)l->data;
-
-		if (strcmp (mScheme.get(), uri_scheme) == 0)
-		{
-			mAppSupportScheme = PR_TRUE;
-		}
-	}
-
-	return NS_OK;
-}
-
-NS_METHOD GContentHandler::Init (void)
+NS_METHOD GContentHandler::Init ()
 {
 	nsresult rv;
 
@@ -241,117 +189,113 @@ NS_METHOD GContentHandler::Init (void)
 	mLauncher->GetMIMEInfo (getter_AddRefs(MIMEInfo));
 	NS_ENSURE_TRUE (MIMEInfo, NS_ERROR_FAILURE);
 
-#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
 	rv = MIMEInfo->GetMIMEType (mMimeType);
 #else
 	rv = MIMEInfo->GetMIMEType (&mMimeType);
 #endif
 
-	mLauncher->GetTargetFile (getter_AddRefs(mTempFile));
-
-	mLauncher->GetSource (getter_AddRefs(mUri));
-	NS_ENSURE_TRUE (mUri, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIURI> uri;
+	mLauncher->GetSource (getter_AddRefs(uri));
+	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
 	
-	rv = mUri->GetSpec (mUrl);
-	rv = mUri->GetScheme (mScheme);
+	uri->GetSpec (mUrl);
 
 	return NS_OK;
 }
 
+static void
+response_cb (GtkWidget *dialog,
+	     int response,
+	     GContentHandler *self)
+{
+	gtk_widget_destroy (dialog);
+
+	if (response > 0)
+	{
+		self->mAction = (ContentAction) response;
+	}
+	else
+	{
+		self->mAction = CONTENT_ACTION_NONE;
+	}
+
+	self->MIMEDoAction ();
+}
+
+static void
+release_cb (GContentHandler *data)
+{
+	NS_RELEASE (data);
+}
+
 NS_METHOD GContentHandler::MIMEConfirmAction ()
 {
 	GtkWidget *dialog;
-	GtkWidget *hbox, *vbox, *label, *image;
-	const char *action_label;
-	char *text;
-	int response;
+	const char *action_label, *primary, *secondary;
+	//int response;
 
 	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (mContext);
-	GtkWindow *parentWindow = GTK_WINDOW (MozillaFindGtkParent(parentDOMWindow));
+	GtkWindow *parentWindow = GTK_WINDOW (EphyUtils::FindGtkParent(parentDOMWindow));
 
 	action_label =  (mAction == CONTENT_ACTION_OPEN) ||
 			(mAction == CONTENT_ACTION_OPEN_TMP) ?
 			GTK_STOCK_OPEN : EPHY_STOCK_DOWNLOAD;
 
-	dialog = gtk_dialog_new_with_buttons
-		("", parentWindow, GTK_DIALOG_NO_SEPARATOR,
-		 _("_Save As..."), CONTENT_ACTION_SAVEAS,
-		 GTK_STOCK_CANCEL, CONTENT_ACTION_NONE,
-		 action_label, mAction, NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), (guint)mAction);
-	
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
-
-	hbox = gtk_hbox_new (FALSE, 6);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
-			    TRUE, TRUE, 0);
-
 	if (mPermission == EPHY_MIME_PERMISSION_UNSAFE && mHelperApp)
 	{
-		text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-					_("Download the unsafe file?"),
-					_("This type of file could potentially damage your documents "
-					  "or invade your privacy. "
-					  "It's not safe to open it directly. You "
-					  "can save it instead."));
+		primary = _("Download the unsafe file?");
+		secondary = _("This type of file could potentially damage "
+			      "your documents or invade your privacy. "
+			      "It's not safe to open it directly. "
+			      "You can save it instead.");
 	}
 	else if (mAction == CONTENT_ACTION_OPEN_TMP)
 	{
-		text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-					_("Open the file in another application?"),
-					_("It's not possible to view this file type directly "
-					  "in the browser. You can open it with another "
-					  "application or save it."));
+		primary = _("Open the file in another application?");
+		secondary = _("It's not possible to view this file type "
+			      "directly in the browser. You can open it with "
+			      "another application or save it.");
 	}
 	else
 	{
-		text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-					_("Download the file?"),
-					_("It's not possible to view this file because there is no "
-					  "application installed that can open it. "
-					  "You can save it instead."));
+		primary = _("Download the file?");
+		secondary = _("It's not possible to view this file because "
+			      "there is no application installed that can open"
+			      " it. You can save it instead.");
 	}
 
-	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
-					  GTK_ICON_SIZE_DIALOG);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-	gtk_widget_show (image);
-	gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
+	dialog = gtk_message_dialog_new
+		(parentWindow,
+		 GTK_DIALOG_DESTROY_WITH_PARENT,
+		 GTK_MESSAGE_WARNING,
+		 GTK_BUTTONS_NONE,	
+		 primary);
 
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+	gtk_message_dialog_format_secondary_text
+		(GTK_MESSAGE_DIALOG (dialog), secondary);
 
-	label = gtk_label_new (NULL);
-	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_label_set_markup (GTK_LABEL (label), text);
-	g_free (text);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Save As..."), CONTENT_ACTION_SAVEAS);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_CANCEL, CONTENT_ACTION_NONE);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       action_label, mAction);
 
-	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-	gtk_widget_show (label);
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
 
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), (guint) mAction);
 
-	if (response == GTK_RESPONSE_DELETE_EVENT)
-	{
-		mAction = CONTENT_ACTION_NONE;
-	}
-	else
-	{
-		mAction = (ContentAction)response;
-	}
-
-	gtk_widget_destroy (dialog);
+	NS_ADDREF (this);
+	g_signal_connect_data (dialog, "response",
+			       G_CALLBACK (response_cb), this,
+			       (GClosureNotify) release_cb, (GConnectFlags) 0);
+	gtk_window_present (GTK_WINDOW (dialog));
 
 	return NS_OK;
 }
 
-NS_METHOD GContentHandler::MIMEDoAction (void)
+NS_METHOD GContentHandler::MIMEInitiateAction (void)
 {
 	gboolean auto_downloads;
 
@@ -359,7 +303,7 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 
 	auto_downloads = eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS);
 
-#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
 	mHelperApp = gnome_vfs_mime_get_default_application (mMimeType.get());
 	mPermission = ephy_embed_shell_check_mime (embed_shell, mMimeType.get());
 #else
@@ -367,8 +311,6 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 	mPermission = ephy_embed_shell_check_mime (embed_shell, mMimeType);
 #endif
 
-	CheckAppSupportScheme ();
-
 	if (auto_downloads)
 	{
 		mAction = CONTENT_ACTION_OPEN;
@@ -378,7 +320,7 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 		mAction = CONTENT_ACTION_OPEN_TMP;
 	}
 
-	if (!mHelperApp || mPermission == EPHY_MIME_PERMISSION_UNSAFE)
+	if (!mHelperApp || mPermission != EPHY_MIME_PERMISSION_SAFE)
 	{
 		mAction = CONTENT_ACTION_DOWNLOAD;
 	}
@@ -387,12 +329,22 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 	{
 		MIMEConfirmAction ();
 	}
+	else
+	{
+		MIMEDoAction ();
+	}
 
+	return NS_OK;
+}
+
+NS_METHOD GContentHandler::MIMEDoAction (void)
+{
 	nsCOMPtr<nsIMIMEInfo> mimeInfo;
 	mLauncher->GetMIMEInfo(getter_AddRefs(mimeInfo));
 	NS_ENSURE_TRUE (mimeInfo, NS_ERROR_FAILURE);
 
-	if (mAction == CONTENT_ACTION_OPEN)
+	if (mAction == CONTENT_ACTION_OPEN ||
+	    mAction == CONTENT_ACTION_OPEN_TMP	)
 	{
 		nsEmbedString desc;
 
@@ -402,7 +354,7 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 		/* HACK we use the application description to ask
 		   MozDownload to open the file when download
 		   is finished */
-#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
 		mimeInfo->SetApplicationDescription (desc);
 #else
 		mimeInfo->SetApplicationDescription (desc.get());
@@ -410,7 +362,7 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 	}
 	else
 	{
-#if MOZILLA_CHECK_VERSION4 (1, 8, MOZILLA_ALPHA, 1)
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
 		mimeInfo->SetApplicationDescription (nsEmbedString ());
 #else
 		mimeInfo->SetApplicationDescription (nsnull);
@@ -419,25 +371,11 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 
 	if (mAction == CONTENT_ACTION_OPEN)
 	{
-		if (mAppSupportScheme)
-		{
-			LaunchHelperApp ();
-		}
-		else
-		{
-			mLauncher->SaveToDisk (nsnull, PR_FALSE);
-		}
+		mLauncher->SaveToDisk (nsnull, PR_FALSE);
 	}
 	else if (mAction == CONTENT_ACTION_OPEN_TMP)
 	{
-		if (mAppSupportScheme)
-		{
-			LaunchHelperApp ();
-		}
-		else
-		{
-			mLauncher->LaunchWithApplication (nsnull, PR_FALSE);
-		}
+		mLauncher->LaunchWithApplication (nsnull, PR_FALSE);
 	}
 	else if (mAction == CONTENT_ACTION_NONE)
 	{
