@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000-2003 Marco Pesenti Gritti, Xan Lopez
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti,
+ *  Copyright (C) 2003 Xan Lopez
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,38 +16,30 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.h,v 1.14.4.3 2004/08/31 12:32:53 chpe Exp $
+ *  $Id: ContentHandler.h,v 1.25 2004/11/16 22:36:10 chpe Exp $
  */
 
-#ifndef __ContentHandler_h
-#define __ContentHandler_h
+#ifndef CONTENT_HANDLER_H
+#define CONTENT_HANDLER_H
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-embed-shell.h"
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include "nsIHelperAppLauncherDialog.h"
-#include "nsIExternalHelperAppService.h"
-#include "nsCExternalHandlerService.h"
-#include "nsIWebProgressListener.h"
 
-#include "nsString.h"
-#include "nsIURI.h"
-#include "nsILocalFile.h"
-
-#include "nsCRT.h"
-#include "nsWeakReference.h"
-
-#include "nsCOMPtr.h"
-#include "nsISupports.h"
-#include "nsError.h"
+#define MOZILLA_STRICT_API
+#include <nsEmbedString.h>
+#undef MOZILLA_STRICT_API
+#include <nsCOMPtr.h>
+#include <nsISupports.h>
+#include <nsIURI.h>
+#include <nsIFile.h>
+#include <nsIHelperAppLauncherDialog.h>
 
 typedef enum
 {
-	CONTENT_ACTION_OPEN,
+	CONTENT_ACTION_OPEN = 1,
 	CONTENT_ACTION_OPEN_TMP,
 	CONTENT_ACTION_DOWNLOAD,
 	CONTENT_ACTION_SAVEAS,
@@ -61,8 +54,6 @@ typedef enum
     {0x9b, 0xeb, 0x93, 0x35, 0xc0, 0x6b, 0xbe, 0xae} \
 }
 
-class nsIFactory;
-
 class GContentHandler : public nsIHelperAppLauncherDialog
 {
   public:
@@ -72,32 +63,27 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	GContentHandler();
 	virtual ~GContentHandler();
 
+	NS_METHOD MIMEDoAction ();
+	ContentAction mAction;
   private:
 
 	NS_METHOD Init ();
-	NS_METHOD LaunchHelperApp ();
 
+	NS_METHOD MIMEInitiateAction ();
 	NS_METHOD MIMEConfirmAction ();
-	NS_METHOD MIMEDoAction ();
-	NS_METHOD CheckAppSupportScheme ();
 
 	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
-	nsCOMPtr<nsIURI> mUri;
-	nsCOMPtr<nsIFile> mTempFile;
 	nsCOMPtr<nsISupports> mContext;
 
-#if MOZILLA_SNAPSHOT < 20
-	char *mMimeType;
-#else
-	nsCString mMimeType;
-#endif	
-	PRBool mAppSupportScheme;
 	GnomeVFSMimeApplication *mHelperApp;
-	ContentAction mAction;
 	EphyMimePermission mPermission;
 
-	nsCString mUrl;
-	nsCString mScheme;
+	nsEmbedCString mUrl;
+#ifdef MOZ_NSIMIMEINFO_NSACSTRING_
+	nsEmbedCString mMimeType;
+#else
+	char *mMimeType;
+#endif
 };
 
-#endif
+#endif /* CONTENT_HANDLER_H */
