@@ -15,19 +15,19 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.37 2004/05/23 01:06:34 marco Exp $
+ *  $Id: EphyBrowser.cpp,v 1.68 2004/11/19 20:16:45 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
+#include "mozilla-config.h"
+
 #include "config.h"
-#endif
 
 #include "EphyBrowser.h"
-#include "GlobalHistory.h"
-#include "ContentHandler.h"
+#include "EphyUtils.h"
 #include "ephy-embed.h"
 #include "ephy-string.h"
 #include "ephy-debug.h"
+#include "print-dialog.h"
 
 #include <gtkmozembed_internal.h>
 #include <unistd.h>
@@ -37,15 +37,14 @@
 #include "nsISimpleEnumerator.h"
 
 #include "nsIContentViewer.h"
-#include "nsIGlobalHistory.h"
 #include "nsIWebBrowserFind.h"
 #include "nsIWebBrowserFocus.h"
 #include "nsICommandManager.h"
 #include "nsIWebBrowserPrint.h"
+#include "nsIDocCharset.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsIDocShellTreeNode.h"
 #include "nsIDocShellTreeOwner.h"
-#include "nsIDocumentCharsetInfo.h"
 #include "nsIWebPageDescriptor.h"
 #include "nsISHEntry.h"
 #include "nsIHistoryEntry.h"
@@ -56,25 +55,54 @@
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMHTMLTextAreaElement.h"
 #include "nsIDOMDocument.h"
+#include "nsIDOM3Document.h"
 #include "nsIDOMEvent.h"
+#include "nsIDOMNSEvent.h"
 #include "nsIDOMEventTarget.h"
+#include "nsIDOMPopupBlockedEvent.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMElement.h"
+#include "nsIDOMWindow2.h"
+#include "nsIDOMDocumentView.h"
+#include "nsIDOMAbstractView.h"
+#define MOZILLA_STRICT_API
+#include "nsEmbedString.h"
+#undef MOZILLA_STRICT_API
+#include "nsMemory.h"
+#include "nsIChannel.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsIServiceManager.h"
+#include "nsIInterfaceRequestor.h"
 
 #ifdef ALLOW_PRIVATE_API
-#include "nsPIDOMWindow.h"
+/* not frozen yet */
+#include "nsIContentPolicy.h"
+/* will never be frozen */
+#include "nsIDocShell.h"
 #include "nsIMarkupDocumentViewer.h"
-#include "nsIChromeEventHandler.h"
-#include "nsIDOMWindowInternal.h"
+#ifdef HAVE_MOZILLA_PSM
+/* not sure about this one: */
+#include <nsITransportSecurityInfo.h>
+/* these are in pipnss/, are they really private? */
+#include <nsISSLStatus.h>
+#include <nsISSLStatusProvider.h>
+#include <nsIX509Cert.h>
+#include <nsICertificateDialogs.h>
+#endif
 #endif
 
-#ifdef ALLOW_PRIVATE_STRINGS
-#include "nsReadableUtils.h"
-#include "nsIDocument.h"
-#include "nsIDeviceContext.h"
-#include "nsIPresContext.h"
-#include "nsIAtom.h"
-#endif
+static PRUnichar DOMLinkAdded[] = { 'D', 'O', 'M', 'L', 'i', 'n', 'k',
+				    'A', 'd', 'd', 'e', 'd', '\0' };
+static PRUnichar DOMPopupBlocked[] = { 'D', 'O', 'M', 'P', 'o', 'p',
+				       'u', 'p', 'B', 'l', 'o', 'c',
+				       'k', 'e', 'd', '\0' };
+static PRUnichar DOMWillOpenModalDialog[] = { 'D', 'O', 'M', 'W', 'i', 'l', 'l',
+					      'O', 'p', 'e', 'n', 'M', 'o', 'd',
+					      'a', 'l', 'D', 'i', 'a', 'l', 'o',
+					      'g', '\0' };
+static PRUnichar DOMModalDialogClosed[] = { 'D', 'O', 'M', 'M', 'o', 'd', 'a',
+					    'l', 'D', 'i', 'a', 'l', 'o', 'g',
+					    'C', 'l', 'o', 's', 'e', 'd', '\0' };
 
 EphyEventListener::EphyEventListener(void)
 : mOwner(nsnull)
@@ -90,94 +118,233 @@ EphyEventListener::~EphyEventListener()
 NS_IMPL_ISUPPORTS1(EphyEventListener, nsIDOMEventListener)
 
 nsresult
-EphyEventListener::Init(EphyEmbed *aOwner)
+EphyEventListener::Init (EphyBrowser *aOwner)
 {
 	mOwner = aOwner;
 	return NS_OK;
 }
 
-nsresult
-EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
-{
-	nsresult result;
-
-	nsCOMPtr<nsIDOMElement> linkElement;
-	linkElement = do_QueryInterface (node);
-	if (!linkElement) return NS_ERROR_FAILURE;
-
-	NS_NAMED_LITERAL_STRING(attr_rel, "rel");
-	nsAutoString value;
-	result = linkElement->GetAttribute (attr_rel, value);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-	if (value.EqualsIgnoreCase("SHORTCUT ICON") ||
-	    value.EqualsIgnoreCase("ICON"))
-	{
-		NS_NAMED_LITERAL_STRING(attr_href, "href");
-		nsAutoString value;
-		result = linkElement->GetAttribute (attr_href, value);
-		if (NS_FAILED (result) || value.IsEmpty()) return NS_ERROR_FAILURE;
-
-		nsCOMPtr<nsIDOMDocument> domDoc;
-		node->GetOwnerDocument(getter_AddRefs(domDoc));
-		NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
-
-		nsCOMPtr<nsIDocument> doc = do_QueryInterface (domDoc);
-		NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
-
-#if MOZILLA_SNAPSHOT > 13
-		nsIURI *uri;
-		uri = doc->GetDocumentURI ();
-#elif MOZILLA_SNAPSHOT > 11
-		nsIURI *uri;
-		uri = doc->GetDocumentURL ();
-#endif
-		if (!uri) return NS_ERROR_FAILURE;
-
-		const nsACString &link = NS_ConvertUTF16toUTF8(value);
-		nsCAutoString favicon_url;
-		result = uri->Resolve (link, favicon_url);
-		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-		
-		char *url = g_strdup (favicon_url.get());
-		g_signal_emit_by_name (mOwner, "ge_favicon", url);
-		g_free (url);
-	}
-
-	return NS_OK;
-}	
-
 NS_IMETHODIMP
 EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 {
 	nsCOMPtr<nsIDOMEventTarget> eventTarget;
-
 	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
 
 	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget);
 	NS_ENSURE_TRUE (node, NS_ERROR_FAILURE);
 
-	HandleFaviconLink (node);
+	nsCOMPtr<nsIDOMElement> linkElement;
+	linkElement = do_QueryInterface (node);
+	if (!linkElement) return NS_ERROR_FAILURE;
+
+	PRUnichar relAttr[] = { 'r', 'e', 'l', '\0' };
+	nsresult rv;
+	nsEmbedString value;
+	rv = linkElement->GetAttribute (nsEmbedString(relAttr), value);
+	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+
+	nsEmbedCString rel;
+	NS_UTF16ToCString (value, NS_CSTRING_ENCODING_UTF8, rel);	
+
+	if (g_ascii_strcasecmp (rel.get(), "SHORTCUT ICON") == 0 ||
+	    g_ascii_strcasecmp (rel.get(), "ICON") == 0)
+	{
+		PRUnichar hrefAttr[] = { 'h', 'r', 'e', 'f', '\0' };
+		nsEmbedString hrefValue;
+		rv = linkElement->GetAttribute (nsEmbedString (hrefAttr), hrefValue);
+		if (NS_FAILED (rv) || !hrefValue.Length()) return NS_ERROR_FAILURE;
+
+		nsEmbedCString link;
+		NS_UTF16ToCString (hrefValue, NS_CSTRING_ENCODING_UTF8, link);
+
+		nsCOMPtr<nsIDOMDocument> domDoc;
+		node->GetOwnerDocument(getter_AddRefs(domDoc));
+		NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIDOMDocumentView> docView (do_QueryInterface (domDoc));
+		NS_ENSURE_TRUE (docView, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIDOMAbstractView> abstractView;
+		docView->GetDefaultView (getter_AddRefs (abstractView));
+
+		nsCOMPtr<nsIDOMWindow> domWin (do_QueryInterface (abstractView));
+		NS_ENSURE_TRUE (domWin, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIDOMWindow> topDomWin;
+		domWin->GetTop (getter_AddRefs (topDomWin));
+
+		nsCOMPtr<nsISupports> domWinAsISupports (do_QueryInterface (domWin));
+		nsCOMPtr<nsISupports> topDomWinAsISupports (do_QueryInterface (topDomWin));
+		/* disallow subframes to set favicon */
+		if (domWinAsISupports != topDomWinAsISupports) return NS_OK;
+
+		nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface (domDoc);
+		NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
+
+		nsEmbedString spec;
+		rv = doc->GetDocumentURI (spec);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIURI> docUri;
+		EphyUtils::NewURI (getter_AddRefs(docUri), spec);
+		NS_ENSURE_TRUE (docUri, NS_ERROR_FAILURE);
+
+		nsEmbedCString faviconUrl;
+		rv = docUri->Resolve (link, faviconUrl);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIURI> favUri;
+		EphyUtils::NewURI (getter_AddRefs (favUri), faviconUrl);
+		NS_ENSURE_TRUE (favUri, NS_ERROR_FAILURE);
+
+		/* check if load is allowed */
+		nsCOMPtr<nsIScriptSecurityManager> secMan
+			(do_GetService("@mozilla.org/scriptsecuritymanager;1"));
+		/* refuse if we can't check */
+		NS_ENSURE_TRUE (secMan, NS_OK);
+
+		rv = secMan->CheckLoadURI(docUri, favUri,
+					  nsIScriptSecurityManager::STANDARD);
+		/* failure means it didn't pass the security check */
+		if (NS_FAILED (rv)) return NS_OK;
+
+		/* security check passed, now check with content policy */
+		nsCOMPtr<nsIContentPolicy> policy =
+			do_GetService("@mozilla.org/layout/content-policy;1");
+		/* refuse if we can't check */
+		NS_ENSURE_TRUE (policy, NS_OK);
+
+#if MOZ_NSICONTENTPOLICY_VARIANT == 2
+		/* FIXME: mozilla tabbrowser.xml passes
+		 * safeGetProperty(event.target, "type") as mimetype guess:
+		 */
+		PRUnichar typeAttr[] = { 't', 'y', 'p', 'e', '\0' };
+		nsEmbedString typeVal;
+		linkElement->GetAttribute (nsEmbedString (typeAttr), typeVal);
+
+		nsEmbedCString cTypeVal;
+		NS_UTF16ToCString (typeVal, NS_CSTRING_ENCODING_UTF8, cTypeVal);
+
+		PRInt16 decision = 0;
+		rv = policy->ShouldLoad (nsIContentPolicy::TYPE_IMAGE,
+					 favUri, docUri, eventTarget,
+					 cTypeVal, nsnull,
+					 &decision);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+		if (decision != nsIContentPolicy::ACCEPT) return NS_OK;
+#else
+		PRBool shouldLoad = PR_FALSE;
+		rv = policy->ShouldLoad (nsIContentPolicy::IMAGE,
+					 favUri, eventTarget,
+					 domWin,
+					 &shouldLoad);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+		if (!shouldLoad) return NS_OK;
+#endif
+
+		/* ok, we accept this as a valid favicon for this site */
+		g_signal_emit_by_name (mOwner->mEmbed, "ge_favicon", faviconUrl.get());
+	}
+
+	return NS_OK;
+}
+
+NS_IMETHODIMP
+EphyPopupBlockEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
+{
+	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
+		do_QueryInterface (aDOMEvent);
+	NS_ENSURE_TRUE (popupEvent, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIURI> popupWindowURI;
+	popupEvent->GetPopupWindowURI (getter_AddRefs (popupWindowURI));
+	NS_ENSURE_TRUE (popupWindowURI, NS_ERROR_FAILURE);
+
+	nsresult rv;
+	nsEmbedCString popupWindowURIString;
+	rv = popupWindowURI->GetSpec (popupWindowURIString);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	nsEmbedString popupWindowFeatures;
+	rv = popupEvent->GetPopupWindowFeatures (popupWindowFeatures);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	nsEmbedCString popupWindowFeaturesString;
+	NS_UTF16ToCString (popupWindowFeatures,
+			   NS_CSTRING_ENCODING_UTF8,
+			   popupWindowFeaturesString);
+
+	g_signal_emit_by_name(mOwner->mEmbed, "ge_popup_blocked",
+			      popupWindowURIString.get(),
+			      popupWindowFeaturesString.get());
+
+	return NS_OK;
+}
+
+NS_IMETHODIMP
+EphyModalAlertEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
+{
+#ifdef MOZ_NSIDOMNSEVENT_GETISTRUSTED
+	NS_ENSURE_TRUE (mOwner, NS_ERROR_FAILURE);
+
+	/* make sure the event is trusted */
+	nsCOMPtr<nsIDOMNSEvent> nsEvent (do_QueryInterface (aDOMEvent));
+	NS_ENSURE_TRUE (nsEvent, NS_ERROR_FAILURE);
+	PRBool isTrusted = PR_FALSE;
+	nsEvent->GetIsTrusted (&isTrusted);
+	if (!isTrusted) return NS_OK;
+
+	nsresult rv;
+	nsEmbedString type;
+	rv = aDOMEvent->GetType (type);
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	nsEmbedCString cType;
+	NS_UTF16ToCString (type, NS_CSTRING_ENCODING_UTF8, cType);
+
+	LOG ("ModalAlertListener event %s", cType.get());
+
+	if (strcmp (cType.get(), "DOMWillOpenModalDialog") == 0)
+	{
+		gboolean retval = FALSE;
+		g_signal_emit_by_name (mOwner->mEmbed, "ge-modal-alert", &retval);
+
+		/* suppress alert */
+		if (retval)
+		{
+			aDOMEvent->PreventDefault ();
+			aDOMEvent->StopPropagation();
+		}
+	}
+	else if (strcmp (cType.get(), "DOMModalDialogClosed") == 0)
+	{
+		g_signal_emit_by_name (mOwner->mEmbed, "ge-modal-alert-closed");
+	}
+#endif
 
 	return NS_OK;
 }
 
 EphyBrowser::EphyBrowser ()
 : mFaviconEventListener(nsnull)
+, mPopupBlockEventListener(nsnull)
+, mModalAlertListener(nsnull)
 , mInitialized(PR_FALSE)
 {
+	LOG ("EphyBrowser ctor (%p)", this)
 }
 
 EphyBrowser::~EphyBrowser ()
 {
+	LOG ("EphyBrowser dtor (%p)", this)
 }
 
 nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 {
-	nsresult rv;
-
 	if (mInitialized) return NS_OK;
 
+	mEmbed = GTK_WIDGET (mozembed);
+
 	gtk_moz_embed_get_nsIWebBrowser (mozembed,
 					 getter_AddRefs(mWebBrowser));
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
@@ -186,6 +353,7 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 	NS_ENSURE_TRUE (mDOMWindow, NS_ERROR_FAILURE);
 
 	/* This will instantiate an about:blank doc if necessary */
+	nsresult rv;
 	nsCOMPtr<nsIDOMDocument> domDocument;
 	rv = mDOMWindow->GetDocument (getter_AddRefs (domDocument));
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
@@ -193,41 +361,55 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 	mFaviconEventListener = new EphyFaviconEventListener();
 	if (!mFaviconEventListener) return NS_ERROR_OUT_OF_MEMORY;
 
-	rv = mFaviconEventListener->Init (EPHY_EMBED (mozembed));
+	rv = mFaviconEventListener->Init (this);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	mPopupBlockEventListener = new EphyPopupBlockEventListener();
+	if (!mPopupBlockEventListener) return NS_ERROR_OUT_OF_MEMORY;
+
+	rv = mPopupBlockEventListener->Init (this);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	mModalAlertListener = new EphyModalAlertEventListener ();
+	if (!mModalAlertListener) return NS_ERROR_OUT_OF_MEMORY;
+
+	rv = mModalAlertListener->Init (this);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
  	rv = GetListener();
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
+	rv = AttachListeners();
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+#ifdef HAVE_MOZILLA_PSM
+	/* FIXME: mozilla sucks! nsWebBrowser already has an instance of this,
+	 * but we cannot get to it!
+	 * See https://bugzilla.mozilla.org/show_bug.cgi?id=94974
+	 */
+	mSecurityInfo = do_CreateInstance(NS_SECURE_BROWSER_UI_CONTRACTID, &rv);
+	NS_ENSURE_TRUE (NS_SUCCEEDED (rv) && mSecurityInfo, NS_ERROR_FAILURE);
+	mSecurityInfo->Init (mDOMWindow);
+#endif
+
 	mInitialized = PR_TRUE;
 
-	return AttachListeners();
+	return NS_OK;
 }
 
 nsresult
 EphyBrowser::GetListener (void)
 {
-  	if (mEventReceiver) return NS_ERROR_FAILURE;
+  	if (mEventTarget) return NS_ERROR_FAILURE;
 
   	nsCOMPtr<nsIDOMWindow> domWindowExternal;
-  	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindowExternal));
+  	mWebBrowser->GetContentDOMWindow (getter_AddRefs(domWindowExternal));
   
-  	nsCOMPtr<nsIDOMWindowInternal> domWindow;
-        domWindow = do_QueryInterface(domWindowExternal);
+  	nsCOMPtr<nsIDOMWindow2> domWindow (do_QueryInterface (domWindowExternal));
+	NS_ENSURE_TRUE (domWindow, NS_ERROR_FAILURE);
 	
-	nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(domWindow));
-	NS_ENSURE_TRUE (piWin, NS_ERROR_FAILURE);
-
-#if MOZILLA_SNAPSHOT >= 18
-  	nsIChromeEventHandler* chromeHandler;
-  	chromeHandler = piWin->GetChromeEventHandler();
-#else
-  	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-  	piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
-#endif
-
-  	mEventReceiver = do_QueryInterface(chromeHandler);
-	NS_ENSURE_TRUE (mEventReceiver, NS_ERROR_FAILURE);
+  	domWindow->GetWindowRoot (getter_AddRefs(mEventTarget));
+	NS_ENSURE_TRUE (mEventTarget, NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
@@ -235,15 +417,17 @@ EphyBrowser::GetListener (void)
 nsresult
 EphyBrowser::AttachListeners(void)
 {
+	NS_ENSURE_TRUE (mEventTarget, NS_ERROR_FAILURE);
+
 	nsresult rv;
-
-	NS_ENSURE_TRUE (mEventReceiver, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIDOMEventTarget> target;
-	target = do_QueryInterface (mEventReceiver);
-
-	rv = target->AddEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
-				      mFaviconEventListener, PR_FALSE);
+	rv = mEventTarget->AddEventListener(nsEmbedString(DOMLinkAdded),
+					    mFaviconEventListener, PR_FALSE);
+	rv |= mEventTarget->AddEventListener(nsEmbedString(DOMPopupBlocked),
+					     mPopupBlockEventListener, PR_FALSE);
+	rv |= mEventTarget->AddEventListener(nsEmbedString(DOMWillOpenModalDialog),
+					     mModalAlertListener, PR_TRUE);
+	rv |= mEventTarget->AddEventListener(nsEmbedString(DOMModalDialogClosed),
+					     mModalAlertListener, PR_TRUE);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	return NS_OK;
@@ -252,58 +436,65 @@ EphyBrowser::AttachListeners(void)
 nsresult
 EphyBrowser::DetachListeners(void)
 {
+	if (!mEventTarget) return NS_OK;
+
 	nsresult rv;
-
-	if (!mEventReceiver) return NS_OK;
-
-	nsCOMPtr<nsIDOMEventTarget> target;
-	target = do_QueryInterface (mEventReceiver);
-	NS_ENSURE_TRUE (target, NS_ERROR_FAILURE);
-
-	rv = target->RemoveEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
-					 mFaviconEventListener, PR_FALSE);
+	rv = mEventTarget->RemoveEventListener(nsEmbedString(DOMLinkAdded),
+					       mFaviconEventListener, PR_FALSE);
+	rv |= mEventTarget->RemoveEventListener(nsEmbedString(DOMPopupBlocked),
+					        mPopupBlockEventListener, PR_FALSE);
+	rv |= mEventTarget->RemoveEventListener(nsEmbedString(DOMWillOpenModalDialog),
+						mModalAlertListener, PR_TRUE);
+	rv |= mEventTarget->RemoveEventListener(nsEmbedString(DOMModalDialogClosed),
+						mModalAlertListener, PR_TRUE);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
 
-nsresult EphyBrowser::Print (nsIPrintSettings *options, PRBool preview)
+nsresult EphyBrowser::Print ()
 {
-	nsresult result;
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
+
+	return  print->Print (nsnull, nsnull);
+}
+
+nsresult EphyBrowser::SetPrintPreviewMode (PRBool previewMode)
+{
+	nsresult rv;
 
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
 	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
 
-	if (!preview)
+	if (previewMode)
 	{
-		result = print->Print (options, nsnull);
+		EmbedPrintInfo *info;
+
+		nsCOMPtr<nsIPrintSettings> settings;
+		print->GetGlobalPrintSettings (getter_AddRefs(settings));
+
+		info = ephy_print_get_print_info ();
+		EphyUtils::CollatePrintSettings (info, settings, TRUE);
+		ephy_print_info_free (info);
+
+		rv = print->PrintPreview (nsnull, mDOMWindow, nsnull);
 	}
 	else
 	{
-		result = print->PrintPreview(options, nsnull, nsnull);
-	}
+		PRBool isPreview = PR_FALSE;
 
-	return result;
-}
+		rv = print->GetDoingPrintPreview(&isPreview);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-nsresult EphyBrowser::PrintPreviewClose (void)
-{
-	nsresult rv;
-	PRBool isPreview = PR_FALSE;
-
-	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
-	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
-
-	rv = print->GetDoingPrintPreview(&isPreview);
-	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
-
-	if (isPreview == PR_TRUE)
-	{
-		rv = print->ExitPrintPreview();
+		if (isPreview == PR_TRUE)
+		{
+			rv = print->ExitPrintPreview();
+		}
 	}
 
 	return rv;
@@ -329,27 +520,15 @@ nsresult EphyBrowser::PrintPreviewNavigate(PRInt16 navType, PRInt32 pageNum)
 	return print->PrintPreviewNavigate(navType, pageNum);
 }
 
-nsresult EphyBrowser::GetPrintSettings (nsIPrintSettings **options)
-{
-	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
-
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
-	NS_ENSURE_TRUE (print, NS_ERROR_FAILURE);
-
-	return print->GetGlobalPrintSettings(options);
-}
-
 nsresult EphyBrowser::GetSHistory (nsISHistory **aSHistory)
 {
-	nsresult result;
-
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIWebNavigation> ContentNav = do_QueryInterface (mWebBrowser);
 	NS_ENSURE_TRUE (ContentNav, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsISHistory> SessionHistory;
-	result = ContentNav->GetSessionHistory (getter_AddRefs (SessionHistory));
+	ContentNav->GetSessionHistory (getter_AddRefs (SessionHistory));
 	NS_ENSURE_TRUE (SessionHistory, NS_ERROR_FAILURE);
 
 	*aSHistory = SessionHistory.get();
@@ -364,7 +543,8 @@ nsresult EphyBrowser::Destroy ()
 
       	mWebBrowser = nsnull;
 	mDOMWindow = nsnull;
-	mEventReceiver = nsnull;
+	mEventTarget = nsnull;
+	mEmbed = nsnull;
 
 	mInitialized = PR_FALSE;
 
@@ -381,68 +561,58 @@ nsresult EphyBrowser::GoToHistoryIndex (PRInt16 index)
 	return ContentNav->GotoIndex (index);
 }
 
-nsresult EphyBrowser::SetZoom (float aZoom, PRBool reflow)
+#ifdef GTKMOZEMBED_BROKEN_RELOAD
+/* Workaround for broken reload with frames, see mozilla bug
+ * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
+ */
+nsresult EphyBrowser::Reload (ReloadType flags)
 {
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	if (reflow)
+	nsCOMPtr<nsISHistory> sessionHistory;
+	GetSHistory (getter_AddRefs (sessionHistory));
+
+	nsCOMPtr<nsIWebNavigation> webNavigation;
+	webNavigation = do_QueryInterface (sessionHistory);
+
+	if (!webNavigation)
 	{
-		nsCOMPtr<nsIContentViewer> contentViewer;	
-		GetContentViewer (getter_AddRefs(contentViewer));
-		NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
-
-		nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
-		NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
-
-		return mdv->SetTextZoom (aZoom);
+		webNavigation = do_QueryInterface (mWebBrowser);
 	}
-	else
+	NS_ENSURE_TRUE (webNavigation, NS_ERROR_FAILURE);
+
+	PRUint32 reloadFlags;
+	switch (flags)
 	{
-		nsCOMPtr<nsIDocShell> DocShell;
-		DocShell = do_GetInterface (mWebBrowser);
-		NS_ENSURE_TRUE (DocShell, NS_ERROR_FAILURE);
-
-		SetZoomOnDocshell (aZoom, DocShell);
-
-		nsCOMPtr<nsIDocShellTreeNode> docShellNode(do_QueryInterface(DocShell));
-		if (docShellNode)
-		{
-			PRInt32 i;
-			PRInt32 n;
-			docShellNode->GetChildCount(&n);
-			for (i=0; i < n; i++) 
-			{
-				nsCOMPtr<nsIDocShellTreeItem> child;
-				docShellNode->GetChildAt(i, getter_AddRefs(child));
-				nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
-				if (childAsShell) 
-				{
-					return SetZoomOnDocshell (aZoom, childAsShell);
-				}
-			}
-		}
+		case RELOAD_FORCE:
+			reloadFlags = nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
+				      nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY;
+			break;
+		case RELOAD_ENCODING_CHANGE:
+			reloadFlags = nsIWebNavigation::LOAD_FLAGS_CHARSET_CHANGE;
+			break;
+		case RELOAD_NORMAL:
+		default:
+			reloadFlags = 0;
+			break;
 	}
 
-	return NS_OK;
+	return webNavigation->Reload (reloadFlags);
 }
-
-nsresult EphyBrowser::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
-{
-	nsCOMPtr<nsIPresContext> PresContext;
-	DocShell->GetPresContext (getter_AddRefs(PresContext));
-	NS_ENSURE_TRUE (PresContext, NS_ERROR_FAILURE);
-
-#if MOZILLA_SNAPSHOT > 13
-	nsIDeviceContext *DeviceContext;
-	DeviceContext = PresContext->DeviceContext();
-	NS_ENSURE_TRUE (DeviceContext, NS_ERROR_FAILURE);
-#else				
-	nsCOMPtr<nsIDeviceContext> DeviceContext;
-	PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
-	NS_ENSURE_TRUE (DeviceContext, NS_ERROR_FAILURE);
 #endif
 
-	return DeviceContext->SetTextZoom (aZoom);
+nsresult EphyBrowser::SetZoom (float aZoom)
+{
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIContentViewer> contentViewer;	
+	GetContentViewer (getter_AddRefs(contentViewer));
+	NS_ENSURE_TRUE (contentViewer, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
+	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
+
+	return mdv->SetTextZoom (aZoom);
 }
 
 nsresult EphyBrowser::GetContentViewer (nsIContentViewer **aViewer)
@@ -476,8 +646,6 @@ nsresult EphyBrowser::GetDocument (nsIDOMDocument **aDOMDocument)
 
 nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 {
-	nsresult result;
-
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
 	/* Use the current target document */
@@ -495,9 +663,10 @@ nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 	webBrowserFocus = do_QueryInterface (mWebBrowser);
 	NS_ENSURE_TRUE (webBrowserFocus, NS_ERROR_FAILURE);
 
+	nsresult rv;
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = webBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
-	if (NS_SUCCEEDED(result) && DOMWindow)
+	rv = webBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	if (NS_SUCCEEDED (rv) && DOMWindow)
 	{
 		return DOMWindow->GetDocument (aDOMDocument);
 	}
@@ -533,14 +702,14 @@ nsresult EphyBrowser::GetSHTitleAtIndex (PRInt32 index, PRUnichar **title)
 					 getter_AddRefs (he));
 	NS_ENSURE_TRUE (he, NS_ERROR_FAILURE);
 
-	nsresult result;
-	result = he->GetTitle (title);
-	NS_ENSURE_TRUE (NS_SUCCEEDED (result) && title, NS_ERROR_FAILURE);
+	nsresult rv;
+	rv = he->GetTitle (title);
+	NS_ENSURE_TRUE (NS_SUCCEEDED (rv) && title, NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
 
-nsresult EphyBrowser::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
+nsresult EphyBrowser::GetSHUrlAtIndex (PRInt32 index, nsACString &url)
 {
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
@@ -557,9 +726,9 @@ nsresult EphyBrowser::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
 	he->GetURI (getter_AddRefs(uri));
 	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
 
-	nsresult result;
-	result = uri->GetSpec(url);
-	NS_ENSURE_TRUE (NS_SUCCEEDED (result) && !url.IsEmpty(), NS_ERROR_FAILURE);
+	nsresult rv;
+	rv = uri->GetSpec(url);
+	NS_ENSURE_TRUE (NS_SUCCEEDED (rv) && url.Length(), NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
@@ -608,50 +777,57 @@ nsresult EphyBrowser::GetPageDescriptor(nsISupports **aPageDescriptor)
 	return NS_OK;
 }
 
-nsresult EphyBrowser::GetDocumentUrl (nsCString &url)
+nsresult EphyBrowser::GetDOMWindow (nsIDOMWindow **aDOMWindow)
 {
+	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
+
+	NS_IF_ADDREF (*aDOMWindow = mDOMWindow);
+
+	return NS_OK;
+}
+
+nsresult EphyBrowser::GetDocumentUrl (nsACString &url)
+{
+	nsresult rv;
+
 	NS_ENSURE_TRUE (mDOMWindow, NS_ERROR_FAILURE);
 
 	nsCOMPtr<nsIDOMDocument> DOMDocument;
 	mDOMWindow->GetDocument (getter_AddRefs(DOMDocument));
 	NS_ENSURE_TRUE (DOMDocument, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
+	nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface(DOMDocument);
 	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *uri;
-	uri = doc->GetDocumentURI ();
-#elif MOZILLA_SNAPSHOT > 11
-	nsIURI *uri;
-	uri = doc->GetDocumentURL ();
-#endif
-	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
+	nsEmbedString docURI;
+	rv = doc->GetDocumentURI (docURI);
+	NS_ENSURE_SUCCESS (rv, rv);
 
-	return uri->GetSpec (url);
+	NS_UTF16ToCString (docURI, NS_CSTRING_ENCODING_UTF8, url);
+
+	return NS_OK;
 }
 
-nsresult EphyBrowser::GetTargetDocumentUrl (nsCString &url)
+nsresult EphyBrowser::GetTargetDocumentUrl (nsACString &url)
 {
+	nsresult rv;
+
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
         nsCOMPtr<nsIDOMDocument> DOMDocument;
 	GetTargetDocument (getter_AddRefs(DOMDocument));
 	NS_ENSURE_TRUE (DOMDocument, NS_ERROR_FAILURE);
 
-        nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
+	nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface(DOMDocument);
 	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *uri;
-	uri = doc->GetDocumentURI ();
-#elif MOZILLA_SNAPSHOT > 11
-	nsIURI *uri;
-	uri = doc->GetDocumentURL ();
-#endif
-	NS_ENSURE_TRUE (uri, NS_ERROR_FAILURE);
+	nsEmbedString docURI;
+	rv = doc->GetDocumentURI (docURI);
+	NS_ENSURE_SUCCESS (rv, rv);
 
-	return uri->GetSpec (url);
+	NS_UTF16ToCString (docURI, NS_CSTRING_ENCODING_UTF8, url);
+
+	return NS_OK;
 }
 
 nsresult EphyBrowser::ForceEncoding (const char *encoding) 
@@ -665,22 +841,20 @@ nsresult EphyBrowser::ForceEncoding (const char *encoding)
 	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
 	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
-	return mdv->SetForceCharacterSet (nsDependentCString(encoding));
+	return mdv->SetForceCharacterSet (nsEmbedCString(encoding));
 }
 
 nsresult EphyBrowser::GetEncoding (nsACString &encoding)
 {
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIDOMDocument> domDoc;
-	GetTargetDocument (getter_AddRefs(domDoc));
-	NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
+	nsCOMPtr<nsIDocCharset> docCharset = do_GetInterface (mWebBrowser);
+	NS_ENSURE_TRUE (docCharset, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
-
-	encoding = doc->GetDocumentCharacterSet ();
-	NS_ENSURE_TRUE (!encoding.IsEmpty(), NS_ERROR_FAILURE);
+	char *charset;
+	docCharset->GetCharset (&charset);
+	encoding = charset;
+	nsMemory::Free (charset);
 
 	return NS_OK;
 }
@@ -696,8 +870,9 @@ nsresult EphyBrowser::GetForcedEncoding (nsACString &encoding)
 	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer);
 	NS_ENSURE_TRUE (mdv, NS_ERROR_FAILURE);
 
-	nsresult result = mdv->GetForceCharacterSet (encoding);
-	NS_ENSURE_SUCCESS (result, NS_ERROR_FAILURE);
+	nsresult rv;
+	rv = mdv->GetForceCharacterSet (encoding);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	return NS_OK;
 }
@@ -740,6 +915,29 @@ nsresult EphyBrowser::GetCommandState (const char *command, PRBool *enabled)
 
 #define NUM_MODIFIED_TEXTFIELDS_REQUIRED	2
 
+PRBool
+EphyBrowser::CompareFormsText (nsAString &aDefaultText, nsAString &aUserText)
+{
+	if (aDefaultText.Length() != aUserText.Length())
+	{
+		return FALSE;
+	}
+
+	/* Mozilla Bug 218277, 195946 and others */
+	const PRUnichar *text = aDefaultText.BeginReading();
+	for (PRUint32 i = 0; i < aDefaultText.Length(); i++)
+	{
+		if (text[i] == 0xa0)
+		{
+			aDefaultText.Replace (i, 1, ' ');
+		}
+	}
+
+	return (memcmp (aDefaultText.BeginReading(),
+		        aUserText.BeginReading(),
+		        aUserText.Length() * sizeof (PRUnichar)) == 0);
+}
+
 nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUint32 *aNumTextFields, PRBool *aHasTextArea)
 {
 	nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(aDomDoc);
@@ -780,14 +978,11 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			nsCOMPtr<nsIDOMHTMLTextAreaElement> areaElement = do_QueryInterface (domNode);
 			if (areaElement)
 			{
-				nsAutoString default_text, user_text;
-				areaElement->GetDefaultValue (default_text);
-				areaElement->GetValue (user_text);
+				nsEmbedString defaultText, userText;
+				areaElement->GetDefaultValue (defaultText);
+				areaElement->GetValue (userText);
 
-				/* Mozilla Bug 218277, 195946 and others */
-				default_text.ReplaceChar(0xa0, ' ');
-
-				if (!user_text.Equals (default_text))
+				if (!CompareFormsText (defaultText, userText))
 				{
 					*aHasTextArea = PR_TRUE;
 					return NS_OK;
@@ -799,31 +994,30 @@ nsresult EphyBrowser::GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUi
 			nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(domNode);
 			if (!inputElement) continue;
 	
-			nsAutoString type;
+			nsEmbedString type;
 			inputElement->GetType(type);
 
-			if (type.EqualsIgnoreCase("text"))
+			nsEmbedCString cType;
+			NS_UTF16ToCString (type, NS_CSTRING_ENCODING_UTF8, cType);
+
+			if (g_ascii_strcasecmp (cType.get(), "text") == 0)
 			{
-				nsAutoString default_text, user_text;
+				nsEmbedString defaultText, userText;
 				PRInt32 max_length;
-				inputElement->GetDefaultValue (default_text);
-				inputElement->GetValue (user_text);
+				inputElement->GetDefaultValue (defaultText);
+				inputElement->GetValue (userText);
 				inputElement->GetMaxLength (&max_length);
 
-				/* Guard against arguably broken forms where
-				 * default_text is longer than maxlength
-				 * (user_text is cropped, default_text is not)
-				 * Mozilla bug 232057
+				/* There are forms for which defaultValue is longer than
+				 * userValue. Mozilla consider this not a bug [see WONTFIXed
+				 * bug 232057], but we need to check for this here.
 				 */
-				if (default_text.Length() > (PRUint32)max_length)
+				if (defaultText.Length() > (PRUint32)max_length)
 				{
-					default_text.Truncate (max_length);
+					defaultText.Cut (max_length, PR_UINT32_MAX);
 				}
 
-				/* Mozilla Bug 218277, 195946 and others */
-				default_text.ReplaceChar(0xa0, ' ');
-
-				if (!user_text.Equals (default_text))
+				if (!CompareFormsText (defaultText, userText))
 				{
 					(*aNumTextFields)++;
 					if (*aNumTextFields >= NUM_MODIFIED_TEXTFIELDS_REQUIRED)
@@ -872,9 +1066,9 @@ nsresult EphyBrowser::GetHasModifiedForms (PRBool *modified)
 		nsCOMPtr<nsIDOMDocument> domDoc;
 		contentViewer->GetDOMDocument (getter_AddRefs (domDoc));
 
-		nsresult result;
-		result = GetDocumentHasModifiedForms (domDoc, &numTextFields, &hasTextArea);
-		if (NS_SUCCEEDED (result) &&
+		nsresult rv;
+		rv = GetDocumentHasModifiedForms (domDoc, &numTextFields, &hasTextArea);
+		if (NS_SUCCEEDED (rv) &&
 		    (numTextFields >= NUM_MODIFIED_TEXTFIELDS_REQUIRED || hasTextArea))
 		{
 			*modified = PR_TRUE;
@@ -884,3 +1078,54 @@ nsresult EphyBrowser::GetHasModifiedForms (PRBool *modified)
 
 	return NS_OK;
 }
+
+nsresult
+EphyBrowser::GetSecurityInfo (PRUint32 *aState, nsACString &aDescription)
+{
+#ifdef HAVE_MOZILLA_PSM
+	NS_ENSURE_TRUE (mSecurityInfo, NS_ERROR_FAILURE);
+
+	nsresult rv;
+	rv = mSecurityInfo->GetState (aState);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	nsEmbedString tooltip;
+	rv = mSecurityInfo->GetTooltipText (tooltip);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	NS_UTF16ToCString (tooltip,
+			   NS_CSTRING_ENCODING_UTF8, aDescription);
+
+	return NS_OK;
+#else
+	return NS_ERROR_NOT_IMPLEMENTED;
+#endif
+}
+
+nsresult
+EphyBrowser::ShowCertificate ()
+{
+#ifdef HAVE_MOZILLA_PSM
+	NS_ENSURE_TRUE (mSecurityInfo, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsISSLStatusProvider> statusProvider (do_QueryInterface (mSecurityInfo));
+	NS_ENSURE_TRUE (statusProvider, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsISSLStatus> SSLStatus;
+	statusProvider->GetSSLStatus (getter_AddRefs (SSLStatus));
+	NS_ENSURE_TRUE (SSLStatus, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIX509Cert> serverCert;
+	SSLStatus->GetServerCert (getter_AddRefs (serverCert));
+	NS_ENSURE_TRUE (serverCert, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsICertificateDialogs> certDialogs (do_GetService (NS_CERTIFICATEDIALOGS_CONTRACTID));
+	NS_ENSURE_TRUE (certDialogs, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIInterfaceRequestor> requestor(do_QueryInterface (mDOMWindow));
+
+	return certDialogs->ViewCert (requestor, serverCert);
+#else
+	return NS_OK;
+#endif
+}
