@@ -36,14 +36,12 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp,v 1.37 2004/10/10 16:03:53 marco Exp $
+ * $Id: EphyHeaderSniffer.cpp,v 1.38 2004/11/02 20:56:55 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "MozDownload.h"
 #include "EphyHeaderSniffer.h"
