@@ -17,23 +17,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSClientAuthDialogs.cpp,v 1.16 2004/10/10 16:03:53 marco Exp $
+ *  $Id: GtkNSSClientAuthDialogs.cpp,v 1.19 2004/11/19 20:24:31 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
-
-#ifdef HAVE_MOZILLA_PSM
 
 #include "EphyUtils.h"
 
 #include <nsIServiceManager.h>
 #include <nsIInterfaceRequestor.h>
 #include <nsIInterfaceRequestorUtils.h>
+#define MOZILLA_STRICT_API
 #include <nsEmbedString.h>
+#undef MOZILLA_STRICT_API
 
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkimage.h>
@@ -284,5 +282,3 @@ GtkNSSClientAuthDialogs::ChooseCertificate (nsIInterfaceRequestor *ctx,
 	gtk_widget_destroy (dialog);
 	return NS_OK;
 }
-
-#endif
