@@ -38,14 +38,12 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.52 2004/10/02 14:25:00 chpe Exp $
+ * $Id: MozDownload.cpp,v 1.54 2004/11/13 19:53:38 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "mozilla-download.h"
 #include "eel-gconf-extensions.h"
@@ -59,7 +57,9 @@
 #include <glib/gi18n.h>
 
 #include <nsIFileURL.h>
+#define MOZILLA_STRICT_API
 #include <nsEmbedString.h>
+#undef MOZILLA_STRICT_API
 
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
 
