@@ -36,21 +36,20 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id$
+ * $Id: MozDownload.h,v 1.19 2004/11/02 20:56:55 chpe Exp $
  */
 
 #ifndef MozDownload_h__
 #define MozDownload_h__
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "mozilla-embed-persist.h"
 #include "downloader-view.h"
 #include "ephy-download.h"
 #include "ephy-embed-shell.h"
 
+#include <nsIDownload.h>
 #include <nsIWebProgressListener.h>
 #include <nsIDOMDocument.h>
 #include <nsIURI.h>
@@ -60,12 +59,6 @@
 #include <nsIRequest.h>
 #include <nsIMIMEInfo.h>
 
-#ifdef HAVE_NSITRANSFER_H
-#include <nsITransfer.h>
-#else
-#include <nsIDownload.h>
-#endif
-
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 /* MozDownload
@@ -96,31 +89,17 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceUri
 				  PRInt32 aMaxSize);
 nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval);
 
-class MozDownload :
-#ifdef HAVE_NSITRANSFER_H
-		    public nsITransfer
-#else
-		    public nsIDownload,
+class MozDownload : public nsIDownload,
                     public nsIWebProgressListener
-#endif
 {
 public:
         MozDownload();
 	virtual ~MozDownload();
     
 	NS_DECL_ISUPPORTS
-	NS_DECL_NSIWEBPROGRESSLISTENER
 	NS_DECL_NSITRANSFER
-#ifndef HAVE_NSITRANSFER_H
 	NS_DECL_NSIDOWNLOAD
-#endif
-
-#ifdef HAVE_NSITRANSFER_H
-	nsresult GetMIMEInfo (nsIMIMEInfo **aMIMEInfo);
-	nsresult GetTargetFile (nsILocalFile **aFile);
-	nsresult GetSource(nsIURI * *aSource);
-	nsresult GetPercentComplete(PRInt32 *aPercentComplete);
-#endif
+	NS_DECL_NSIWEBPROGRESSLISTENER
 
 	virtual void Cancel();
 	virtual void Pause();
