@@ -16,23 +16,24 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozRegisterComponents.cpp,v 1.28 2004/07/30 19:56:08 chpe Exp $
+ *  $Id: MozRegisterComponents.cpp,v 1.30 2004/11/19 20:13:44 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ContentHandler.h"
-#include "FilePicker.h"
 #include "GlobalHistory.h"
 #include "PrintingPromptService.h"
 #include "MozDownload.h"
 #include "EphyAboutRedirector.h"
 #include "EphyContentPolicy.h"
 
+#ifdef ENABLE_FILEPICKER
+#include "FilePicker.h"
+#endif
+
 #ifdef HAVE_MOZILLA_PSM
 #include "GtkNSSClientAuthDialogs.h"
 #include "GtkNSSDialogs.h"
@@ -51,13 +52,16 @@
 #include <glib/gmessages.h>
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphyAboutRedirector)
-NS_GENERIC_FACTORY_CONSTRUCTOR(MozDownload)	
-NS_GENERIC_FACTORY_CONSTRUCTOR(GFilePicker)
+NS_GENERIC_FACTORY_CONSTRUCTOR(MozDownload)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GContentHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphyContentPolicy)
 
+#ifdef ENABLE_FILEPICKER
+NS_GENERIC_FACTORY_CONSTRUCTOR(GFilePicker)
+#endif
+
 #ifdef HAVE_MOZILLA_PSM
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
@@ -88,12 +92,14 @@ static const nsModuleComponentInfo sAppComps[] = {
 		NS_DOWNLOAD_CONTRACTID,
 		MozDownloadConstructor
 	},
+#ifdef ENABLE_FILEPICKER
 	{
 		G_FILEPICKER_CLASSNAME,
 		G_FILEPICKER_CID,
 		G_FILEPICKER_CONTRACTID,
 		GFilePickerConstructor
 	},
+#endif
 #ifdef HAVE_MOZILLA_PSM
 	{
 		GTK_NSSCLIENTAUTHDIALOGS_CLASSNAME,
