@@ -21,12 +21,10 @@
 
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 
-   $Id: eel-gconf-extensions.c,v 1.11 2004/04/07 10:38:47 chpe Exp $
+   $Id: eel-gconf-extensions.c,v 1.12 2004/11/02 20:57:20 chpe Exp $
 */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "eel-gconf-extensions.h"
 
