@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,20 +16,20 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-chooser.c,v 1.6 2004/01/24 11:36:04 chpe Exp $
+ *  $Id: ephy-file-chooser.c,v 1.17 2004/11/29 13:43:24 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-file-chooser.h"
+#include "ephy-file-helpers.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-state.h"
 #include "ephy-debug.h"
 
 #include <gtk/gtkstock.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <glib/gi18n.h>
 
 #define EPHY_FILE_CHOOSER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_FILE_CHOOSER, EphyFileChooserPrivate))
 
@@ -54,7 +54,7 @@ ephy_file_chooser_get_type (void)
 {
 	static GType type = 0;
 
-	if (type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -96,13 +96,28 @@ static void
 ephy_file_chooser_init (EphyFileChooser *dialog)
 {
 	dialog->priv = EPHY_FILE_CHOOSER_GET_PRIVATE (dialog);
+}
 
-	dialog->priv->persist_key = NULL;
+static GObject *
+ephy_file_chooser_constructor (GType type,
+			       guint n_construct_properties,
+			       GObjectConstructParam *construct_params)
 
-	ephy_state_add_window (GTK_WIDGET (dialog), "file_chooser",
-			       400, 300,
-			       EPHY_STATE_WINDOW_SAVE_SIZE |
-			       EPHY_STATE_WINDOW_SAVE_POSITION);
+{
+	GObject *object;
+	char *downloads_dir;
+
+	object = parent_class->constructor (type, n_construct_properties,
+					    construct_params);
+
+	downloads_dir = ephy_file_downloads_dir ();
+	gtk_file_chooser_add_shortcut_folder
+		(GTK_FILE_CHOOSER (object), downloads_dir, NULL);
+	g_free (downloads_dir);
+
+	gtk_window_set_icon_name (GTK_WINDOW (object), "web-browser");
+
+	return object;
 }
 
 static void
@@ -132,22 +147,22 @@ ephy_file_chooser_set_persist_key (EphyFileChooser *dialog, const char *key)
 		converted = g_filename_from_utf8
 			(dir, -1, NULL, NULL, NULL);
 
-		expanded = gnome_vfs_expand_initial_tilde (converted);
-
-		if (expanded != NULL)
+		if (converted != NULL)
 		{
+			expanded = gnome_vfs_expand_initial_tilde (converted);
+
 			gtk_file_chooser_set_current_folder
 				(GTK_FILE_CHOOSER (dialog), expanded);
+
+			g_free (expanded);
+			g_free (converted);
 		}
 
-		g_free (expanded);
-		g_free (converted);
 		g_free (dir);
 	}
 
 	g_signal_connect (dialog, "current-folder-changed",
 			  G_CALLBACK (current_folder_changed_cb), dialog);
-
 }
 
 const char *
@@ -158,6 +173,64 @@ ephy_file_chooser_get_persist_key (EphyFileChooser *dialog)
 	return dialog->priv->persist_key;
 }
 
+GtkFileFilter *
+ephy_file_chooser_add_pattern_filter (EphyFileChooser *dialog,
+				      const char *title,
+				      const char *first_pattern,
+				      ...)
+{
+	GtkFileFilter *filth;
+	va_list args;
+	const char *pattern;
+
+	filth = gtk_file_filter_new ();
+
+	va_start (args, first_pattern);
+
+	pattern = first_pattern;
+	while (pattern != NULL)
+	{
+		gtk_file_filter_add_pattern (filth, pattern);
+		pattern = va_arg (args, const char *);
+	}
+	va_end (args);
+
+	gtk_file_filter_set_name (filth, title);
+
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filth);
+
+	return filth;
+}
+
+GtkFileFilter *
+ephy_file_chooser_add_mime_filter (EphyFileChooser *dialog,
+				   const char *title,
+				   const char *first_mimetype,
+				   ...)
+{
+	GtkFileFilter *filth;
+	va_list args;
+	const char *mimetype;
+
+	filth = gtk_file_filter_new ();
+
+	va_start (args, first_mimetype);
+
+	mimetype = first_mimetype;
+	while (mimetype != NULL)
+	{
+		gtk_file_filter_add_mime_type (filth, mimetype);
+		mimetype = va_arg (args, const char *);
+	}
+	va_end (args);
+
+	gtk_file_filter_set_name (filth, title);
+
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filth);
+
+	return filth;
+}
+
 static void
 ephy_file_chooser_set_property (GObject *object,
 				guint prop_id,
@@ -197,6 +270,7 @@ ephy_file_chooser_class_init (EphyFileChooserClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->constructor = ephy_file_chooser_constructor;
 	object_class->finalize = ephy_file_chooser_finalize;
 	object_class->get_property = ephy_file_chooser_get_property;
 	object_class->set_property = ephy_file_chooser_set_property;
@@ -216,9 +290,13 @@ EphyFileChooser	*
 ephy_file_chooser_new (const char *title,
 		       GtkWidget *parent,
 		       GtkFileChooserAction action,
-		       const char *persist_key)
+		       const char *persist_key,
+		       EphyFileFilterDefault default_filter)
 {
 	EphyFileChooser *dialog;
+	GtkFileFilter *filter[EPHY_FILE_FILTER_LAST];
+
+	g_return_val_if_fail (default_filter >= 0 && default_filter <= EPHY_FILE_FILTER_LAST, NULL);
 
 	dialog = EPHY_FILE_CHOOSER (g_object_new (EPHY_TYPE_FILE_CHOOSER,
 						  "title", title,
@@ -236,23 +314,63 @@ ephy_file_chooser_new (const char *title,
 		ephy_file_chooser_set_persist_key (dialog, persist_key);
 	}
 
-	if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN	    ||
+	    action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER ||
+	    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER)
 	{
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_OPEN, EPHY_RESPONSE_OPEN,
+					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 					NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-						 EPHY_RESPONSE_OPEN);
+						 GTK_RESPONSE_ACCEPT);
 	}
 	else if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
 	{
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_SAVE, EPHY_RESPONSE_SAVE,
+					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 					NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-						 EPHY_RESPONSE_SAVE);
+						 GTK_RESPONSE_ACCEPT);
+	}
+
+	if (default_filter != EPHY_FILE_FILTER_NONE)
+	{
+		filter[EPHY_FILE_FILTER_ALL_SUPPORTED] =
+			ephy_file_chooser_add_mime_filter
+				(dialog,
+				 _("All supported types"),
+				 "text/html",
+				 "application/xhtml+xml",
+				 "text/xml",
+				 "image/png",
+				 "image/jpeg",
+				 "image/gif",
+				 NULL);
+
+		filter[EPHY_FILE_FILTER_WEBPAGES] =
+			ephy_file_chooser_add_mime_filter
+				(dialog, _("Web pages"),
+				 "text/html",
+				 "application/xhtml+xml",
+				 "text/xml",
+				 NULL);
+
+		filter[EPHY_FILE_FILTER_IMAGES] =
+			ephy_file_chooser_add_mime_filter
+				(dialog, _("Images"),
+				 "image/png",
+				 "image/jpeg",
+				 "image/gif",
+				 NULL);
+
+		filter[EPHY_FILE_FILTER_ALL] =
+			ephy_file_chooser_add_pattern_filter
+				(dialog, _("All files"), "*", NULL);
+
+		gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog),
+					     filter[default_filter]);
 	}
 
 	if (parent != NULL)
