@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-gui.c,v 1.25 2004/10/26 20:02:22 chpe Exp $
+ *  $Id: ephy-gui.c,v 1.29 2004/11/09 20:32:01 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-gui.h"
 #include "eel-gconf-extensions.h"
@@ -38,18 +36,19 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtktreeselection.h>
 
-/* Styles for tab labels */
-GtkStyle *loading_text_style = NULL;
-GtkStyle *new_text_style = NULL;
-
-static void
-sanitize_popup_position (GtkWidget *widget, GtkMenu *menu, gint *x, gint *y)
+void
+ephy_gui_sanitise_popup_position (GtkMenu *menu,
+				  GtkWidget *widget,
+				  gint *x,
+				  gint *y)
 {
 	GdkScreen *screen = gtk_widget_get_screen (widget);
 	gint monitor_num;
 	GdkRectangle monitor;
 	GtkRequisition req;
 
+	g_return_if_fail (widget != NULL);
+
 	gtk_widget_size_request (GTK_WIDGET (menu), &req);
 
 	monitor_num = gdk_screen_get_monitor_at_point (screen, *x, *y);
@@ -73,12 +72,17 @@ ephy_gui_menu_position_tree_selection (GtkMenu   *menu,
 	GtkTreeView *tree_view = GTK_TREE_VIEW (user_data);
 	GtkWidget *widget = GTK_WIDGET (user_data);
 	GtkRequisition req;
+	GdkRectangle visible;
 
 	gtk_widget_size_request (GTK_WIDGET (menu), &req);
 	gdk_window_get_origin (widget->window, x, y);
 
 	*x += (widget->allocation.width - req.width) / 2;
 
+	/* Add on height for the treeview title */
+	gtk_tree_view_get_visible_rect (tree_view, &visible);
+	*y += widget->allocation.height - visible.height;
+
 	selection = gtk_tree_view_get_selection (tree_view);
 	selected_rows = gtk_tree_selection_get_selected_rows (selection, &model);
 	if (selected_rows)
@@ -88,17 +92,17 @@ ephy_gui_menu_position_tree_selection (GtkMenu   *menu,
 		gtk_tree_view_get_cell_area (tree_view, selected_rows->data,
 					     NULL, &cell_rect);
 
-		*y += CLAMP (cell_rect.y + cell_rect.height, 0, widget->allocation.height);
+		*y += CLAMP (cell_rect.y + cell_rect.height, 0, visible.height);
 
 		g_list_foreach (selected_rows, (GFunc)gtk_tree_path_free, NULL);
 		g_list_free (selected_rows);
 	}
 
-	sanitize_popup_position (widget, menu, x, y);
+	ephy_gui_sanitise_popup_position (menu, widget, x, y);
 }
 
 /**
- * gul_gui_menu_position_under_widget:
+ * ephy_gui_menu_position_under_widget:
  */
 void
 ephy_gui_menu_position_under_widget (GtkMenu   *menu,
@@ -127,14 +131,14 @@ ephy_gui_menu_position_under_widget (GtkMenu   *menu,
 
 	*y += w->allocation.y + w->allocation.height;
 
-	sanitize_popup_position (w, menu, x, y);
+	ephy_gui_sanitise_popup_position (menu, w, x, y);
 }
 
 gboolean
 ephy_gui_confirm_overwrite_file (GtkWidget *parent, const char *filename)
 {
 	GtkWidget *dialog;
-	char  *converted;
+	char *display_name;
 	gboolean retval;
 
 	if (filename == NULL) return FALSE;
@@ -144,15 +148,14 @@ ephy_gui_confirm_overwrite_file (GtkWidget *parent, const char *filename)
 		return TRUE;
 	}
 
-	converted = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-	if (converted == NULL) return FALSE;
+	display_name = g_filename_display_name (filename);
 
 	dialog = gtk_message_dialog_new
 		(parent ? GTK_WINDOW (parent) : NULL,
 		 GTK_DIALOG_MODAL,
 		 GTK_MESSAGE_WARNING,
 		 GTK_BUTTONS_CANCEL,
-		 _("A file %s already exists."), converted);
+		 _("A file %s already exists."), display_name);
 
 	gtk_message_dialog_format_secondary_text
 		(GTK_MESSAGE_DIALOG (dialog),
@@ -171,7 +174,7 @@ ephy_gui_confirm_overwrite_file (GtkWidget *parent, const char *filename)
 
 	gtk_widget_destroy (dialog);
 
-	g_free (converted);
+	g_free (display_name);
 
 	return retval;
 }
