@@ -39,8 +39,8 @@ G_BEGIN_DECLS
 #define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
 #define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
 #define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
-#define g_marshal_value_peek_enum(v)     (v)->data[0].v_int
-#define g_marshal_value_peek_flags(v)    (v)->data[0].v_uint
+#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
+#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
 #define g_marshal_value_peek_float(v)    (v)->data[0].v_float
 #define g_marshal_value_peek_double(v)   (v)->data[0].v_double
 #define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
@@ -51,7 +51,7 @@ G_BEGIN_DECLS
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* INT:STRING (./ephy-marshal.list:1) */
+/* INT:STRING (../../lib/ephy-marshal.list:1) */
 extern void ephy_marshal_INT__STRING (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
@@ -96,7 +96,7 @@ ephy_marshal_INT__STRING (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT (./ephy-marshal.list:2) */
+/* BOOLEAN:OBJECT (../../lib/ephy-marshal.list:2) */
 extern void ephy_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -141,7 +141,7 @@ ephy_marshal_BOOLEAN__OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,STRING (./ephy-marshal.list:3) */
+/* BOOLEAN:STRING,STRING (../../lib/ephy-marshal.list:3) */
 extern void ephy_marshal_BOOLEAN__STRING_STRING (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -188,7 +188,50 @@ ephy_marshal_BOOLEAN__STRING_STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:ENUM,INT,BOOLEAN (./ephy-marshal.list:4) */
+/* BOOLEAN:VOID (../../lib/ephy-marshal.list:4) */
+extern void ephy_marshal_BOOLEAN__VOID (GClosure     *closure,
+                                        GValue       *return_value,
+                                        guint         n_param_values,
+                                        const GValue *param_values,
+                                        gpointer      invocation_hint,
+                                        gpointer      marshal_data);
+void
+ephy_marshal_BOOLEAN__VOID (GClosure     *closure,
+                            GValue       *return_value,
+                            guint         n_param_values,
+                            const GValue *param_values,
+                            gpointer      invocation_hint,
+                            gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__VOID) (gpointer     data1,
+                                                  gpointer     data2);
+  register GMarshalFunc_BOOLEAN__VOID callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 1);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__VOID) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
+/* VOID:ENUM,INT,BOOLEAN (../../lib/ephy-marshal.list:5) */
 extern void ephy_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -233,7 +276,7 @@ ephy_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT (./ephy-marshal.list:5) */
+/* VOID:INT,INT (../../lib/ephy-marshal.list:6) */
 extern void ephy_marshal_VOID__INT_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -276,7 +319,7 @@ ephy_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (./ephy-marshal.list:6) */
+/* VOID:POINTER,INT (../../lib/ephy-marshal.list:7) */
 extern void ephy_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -319,7 +362,7 @@ ephy_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,BOOLEAN (./ephy-marshal.list:7) */
+/* VOID:POINTER,BOOLEAN (../../lib/ephy-marshal.list:8) */
 extern void ephy_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -362,7 +405,7 @@ ephy_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (./ephy-marshal.list:8) */
+/* VOID:POINTER,POINTER (../../lib/ephy-marshal.list:9) */
 extern void ephy_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -405,7 +448,7 @@ ephy_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./ephy-marshal.list:9) */
+/* VOID:STRING,INT (../../lib/ephy-marshal.list:10) */
 extern void ephy_marshal_VOID__STRING_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -448,7 +491,7 @@ ephy_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT,INT (./ephy-marshal.list:10) */
+/* VOID:STRING,INT,INT (../../lib/ephy-marshal.list:11) */
 extern void ephy_marshal_VOID__STRING_INT_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -493,7 +536,7 @@ ephy_marshal_VOID__STRING_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (./ephy-marshal.list:11) */
+/* VOID:STRING,STRING (../../lib/ephy-marshal.list:12) */
 extern void ephy_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -536,7 +579,7 @@ ephy_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,BOOLEAN (./ephy-marshal.list:12) */
+/* VOID:STRING,BOOLEAN (../../lib/ephy-marshal.list:13) */
 extern void ephy_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
