@@ -16,12 +16,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-filter.c,v 1.7 2004/10/23 16:47:27 chpe Exp $
+ *  $Id: ephy-node-filter.c,v 1.8 2004/11/02 20:57:20 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <stdlib.h>
 #include <string.h>
