@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-string.c,v 1.13 2004/03/17 09:02:35 marco Exp $
+ *  $Id: ephy-string.c,v 1.16 2004/11/02 20:57:20 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-string.h"
 
