@@ -18,12 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-entry.c,v 1.42 2004/03/17 09:02:37 marco Exp $
+ *  $Id: ephy-location-entry.c,v 1.46 2004/11/02 20:57:20 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-tree-model-node.h"
 #include "ephy-location-entry.h"
@@ -46,7 +44,6 @@ struct _EphyLocationEntryPrivate
 	GtkWidget *entry;
 	char *before_completion;
 	gboolean user_changed;
-	GtkWidget *shown_widget;
 
 	guint text_col;
 	guint action_col;
@@ -54,15 +51,19 @@ struct _EphyLocationEntryPrivate
 	guint relevance_col;
 };
 
-static char *web_prefixes [] =
+static const struct
 {
-	"http://www.",
-        "http://",
-        "https://www.",
-        "https://",
-        "www."
+	const char *prefix;
+	int len;
+}
+web_prefixes [] =
+{
+	{ "http://www.", 11 },
+	{ "http://", 7 },
+	{ "https://www.", 12 },
+	{ "https://", 8 },
+	{ "www.", 4 }
 };
-static int n_web_prefixes = G_N_ELEMENTS (web_prefixes);
 
 static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
@@ -93,9 +94,9 @@ enum
 GType
 ephy_location_entry_get_type (void)
 {
-	static GType ephy_location_entry_type = 0;
+	static GType type = 0;
 
-	if (ephy_location_entry_type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -110,12 +111,12 @@ ephy_location_entry_get_type (void)
 			(GInstanceInitFunc) ephy_location_entry_init
 		};
 
-		ephy_location_entry_type = g_type_register_static (GTK_TYPE_TOOL_ITEM,
-							           "EphyLocationEntry",
-							           &our_info, 0);
+		type = g_type_register_static (GTK_TYPE_TOOL_ITEM,
+					       "EphyLocationEntry",
+					       &our_info, 0);
 	}
 
-	return ephy_location_entry_type;
+	return type;
 }
 
 static gboolean
@@ -156,24 +157,6 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 	g_type_class_add_private (object_class, sizeof (EphyLocationEntryPrivate));
 }
 
-static void
-ephy_location_entry_activation_finished (EphyLocationEntry *entry)
-{
-	if (entry->priv->shown_widget)
-	{
-		gtk_widget_hide (entry->priv->shown_widget);
-		entry->priv->shown_widget = NULL;
-	}
-}
-
-static gboolean
-location_focus_out_cb (GtkWidget *leidget, GdkEventFocus *event, EphyLocationEntry *le)
-{
-	ephy_location_entry_activation_finished (le);
-
-	return FALSE;
-}
-
 static void
 editable_changed_cb (GtkEditable *editable, EphyLocationEntry *e)
 {
@@ -236,7 +219,7 @@ completion_func (GtkEntryCompletion *completion,
 		 GtkTreeIter *iter,
 		 gpointer data)
 {
-	int i;
+	int i, len_key, len_prefix;
 	char *item = NULL;
 	char *keywords = NULL;
 	gboolean ret = FALSE;
@@ -250,7 +233,8 @@ completion_func (GtkEntryCompletion *completion,
 			    le->priv->keywords_col, &keywords,
 			    -1);
 
-	if (!strncmp (key, item, strlen (key)))
+	len_key = strlen (key);
+	if (!strncmp (key, item, len_key))
 	{
 		ret = TRUE;
 	}
@@ -260,21 +244,15 @@ completion_func (GtkEntryCompletion *completion,
 	}
 	else
 	{
-		for (i = 0; i < n_web_prefixes; i++)
+		for (i = 0; i < G_N_ELEMENTS (web_prefixes); i++)
 		{
-			char *key_prefixed;
-
-			key_prefixed = g_strconcat (web_prefixes[i], key, NULL);
-
-			if (!strncmp (key_prefixed, item, strlen (key_prefixed)))
+			len_prefix = web_prefixes[i].len;
+			if (!strncmp (web_prefixes[i].prefix, item, len_prefix) &&
+			    !strncmp (key, item + len_prefix, len_key))
 			{
-				g_free (key_prefixed);
-
 				ret = TRUE;
 				break;
 			}
-
-			g_free (key_prefixed);
 		}
 	}
 
@@ -382,15 +360,10 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	le->priv = p;
 
 	p->user_changed = TRUE;
-	p->shown_widget = NULL;
 
 	ephy_location_entry_construct_contents (le);
 
 	gtk_tool_item_set_expand (GTK_TOOL_ITEM (le), TRUE);
-
-	g_signal_connect (le->priv->entry,
-	       		  "focus_out_event",
-                          G_CALLBACK (location_focus_out_cb), le);
 }
 
 GtkWidget *
@@ -480,17 +453,7 @@ ephy_location_entry_get_location (EphyLocationEntry *le)
 void
 ephy_location_entry_activate (EphyLocationEntry *le)
 {
-	GtkWidget *toplevel, *widget;
-
-	for (widget = GTK_WIDGET (le); widget != NULL; widget = widget->parent)
-	{
-		if (!GTK_WIDGET_VISIBLE (widget))
-		{
-			le->priv->shown_widget = widget;
-			gtk_widget_show (le->priv->shown_widget);
-			break;
-		}
-	}
+	GtkWidget *toplevel;
 
 	toplevel = gtk_widget_get_toplevel (le->priv->entry);
 
