@@ -15,10 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-search-entry.c,v 1.5 2003/12/16 17:29:41 marco Exp $
+ *  $Id: ephy-search-entry.c,v 1.9 2004/11/02 20:57:20 chpe Exp $
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <gtk/gtklabel.h>
 #include <glib/gi18n.h>
@@ -51,9 +51,9 @@ static guint ephy_search_entry_signals[LAST_SIGNAL] = { 0 };
 GType
 ephy_search_entry_get_type (void)
 {
-	static GType ephy_search_entry_type = 0;
+	static GType type = 0;
 
-	if (ephy_search_entry_type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -68,12 +68,12 @@ ephy_search_entry_get_type (void)
 			(GInstanceInitFunc) ephy_search_entry_init
 		};
 
-		ephy_search_entry_type = g_type_register_static (GTK_TYPE_ENTRY,
-							         "EphySearchEntry",
-							         &our_info, 0);
+		type = g_type_register_static (GTK_TYPE_ENTRY,
+					       "EphySearchEntry",
+					       &our_info, 0);
 	}
 
-	return ephy_search_entry_type;
+	return type;
 }
 
 static void
@@ -139,9 +139,6 @@ ephy_search_entry_init (EphySearchEntry *entry)
 {
 	entry->priv = EPHY_SEARCH_ENTRY_GET_PRIVATE (entry);
 
-	entry->priv->clearing = FALSE;
-	entry->priv->timeout = 0;
-
 	g_signal_connect (G_OBJECT (entry),
 			  "destroy",
 			  G_CALLBACK (ephy_search_entry_destroy_cb),
