@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmark-properties.c,v 1.32.4.2 2004/11/01 14:08:33 chpe Exp $
+ *  $Id: ephy-bookmark-properties.c,v 1.38 2004/11/02 20:57:33 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-bookmarksbar-model.h"
 #include "ephy-bookmark-properties.h"
@@ -79,9 +77,9 @@ static GObjectClass *parent_class = NULL;
 GType
 ephy_bookmark_properties_get_type (void)
 {
-	static GType ephy_bookmark_properties_type = 0;
+	static GType type = 0;
 
-	if (ephy_bookmark_properties_type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -96,12 +94,12 @@ ephy_bookmark_properties_get_type (void)
 			(GInstanceInitFunc) ephy_bookmark_properties_init
 		};
 
-		ephy_bookmark_properties_type = g_type_register_static (GTK_TYPE_DIALOG,
-							                "EphyBookmarkProperties",
-							                &our_info, 0);
+		type = g_type_register_static (GTK_TYPE_DIALOG,
+					       "EphyBookmarkProperties",
+					       &our_info, 0);
 	}
 
-	return ephy_bookmark_properties_type;
+	return type;
 }
 
 static void
@@ -280,30 +278,28 @@ set_window_icon (EphyBookmarkProperties *editor)
 	GdkPixbuf *icon = NULL;
 
 	cache = EPHY_FAVICON_CACHE
-		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
+		(ephy_embed_shell_get_favicon_cache (embed_shell));
 	icon_location = ephy_node_get_property_string
 		(editor->priv->bookmark, EPHY_NODE_BMK_PROP_ICON);
 
 	LOG ("Get favicon for %s", icon_location ? icon_location : "None")
 
-	if (icon_location)
+	if (icon_location != NULL)
 	{
 		icon = ephy_favicon_cache_get (cache, icon_location);
 	}
 
-	else
-	{
-		icon = gtk_widget_render_icon (GTK_WIDGET (editor),
-					       GTK_STOCK_PROPERTIES,
-					       GTK_ICON_SIZE_MENU,
-					       NULL);
-	}
-
 	if (icon != NULL)
 	{
 		gtk_window_set_icon (GTK_WINDOW (editor), icon);
 		g_object_unref (icon);
 	}
+	else
+	{
+		gtk_window_set_icon_name (GTK_WINDOW (editor),
+					  GTK_STOCK_PROPERTIES);
+	}
+
 }
 
 static void
@@ -427,8 +423,6 @@ static void
 ephy_bookmark_properties_init (EphyBookmarkProperties *editor)
 {
 	editor->priv = EPHY_BOOKMARK_PROPERTIES_GET_PRIVATE (editor);
-
-	editor->priv->bookmark = NULL;
 }
 
 GtkWidget *
