@@ -16,18 +16,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-menu.c,v 1.29 2004/08/23 12:08:01 chpe Exp $
+ *  $Id: ephy-bookmarks-menu.c,v 1.37 2004/11/02 20:57:34 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-bookmarks-menu.h"
 #include "ephy-bookmark-action.h"
 #include "ephy-shell.h"
 #include "ephy-node-common.h"
 #include "ephy-string.h"
+#include "ephy-marshal.h"
+#include "ephy-gui.h"
 #include "ephy-debug.h"
 
 #include <glib/gprintf.h>
@@ -40,7 +40,7 @@
 struct _EphyBookmarksMenuPrivate
 {
 	GtkUIManager *manager;
-	EphyWindow *window;
+	char *path;
 	EphyBookmarks *bookmarks;
 	GtkActionGroup *bmk_actions;
 	GtkActionGroup *folder_actions;
@@ -50,8 +50,6 @@ struct _EphyBookmarksMenuPrivate
 	gboolean needs_update;
 };
 
-#define BOOKMARKS_MENU_PATH "/menubar/BookmarksMenu"
-
 /* 14 = strlen ("0000000000000000") - strlen ("%x")
  * FIXME: for 32bit, 6 is sufficient -> use some #if magic?
  */
@@ -71,13 +69,23 @@ struct _EphyBookmarksMenuPrivate
 
 #define GAZILLION	200
 #define UPDATE_DELAY	5000 /* ms */
+#define LABEL_WIDTH_CHARS	32
 
 enum
 {
 	PROP_0,
-	PROP_WINDOW
+	PROP_PATH,
+	PROP_UI_MANAGER
 };
 
+enum
+{
+	OPEN,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void ephy_bookmarks_menu_class_init (EphyBookmarksMenuClass *klass);
 static void ephy_bookmarks_menu_init	   (EphyBookmarksMenu *menu);
 
@@ -88,7 +96,7 @@ ephy_bookmarks_menu_get_type (void)
 {
         static GType type = 0;
 
-        if (type == 0)
+        if (G_UNLIKELY (type == 0))
         {
                 static const GTypeInfo our_info =
                 {
@@ -111,6 +119,21 @@ ephy_bookmarks_menu_get_type (void)
         return type;
 }
 
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+                  GtkAction *action,
+                  GtkWidget *proxy)
+{
+        if (GTK_IS_MENU_ITEM (proxy))
+        {
+		GtkLabel *label;
+
+		label = (GtkLabel *) ((GtkBin *) proxy)->child;
+		gtk_label_set_width_chars (label, LABEL_WIDTH_CHARS);
+		gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+        }
+}
+
 static void
 remove_action (gpointer idptr,
 	       GtkActionGroup *action_group)
@@ -162,7 +185,17 @@ open_bookmark_cb (GtkAction *action,
 		  const char *location,
 		  EphyBookmarksMenu *menu)
 {
-	ephy_window_load_url (menu->priv->window, location);
+	g_signal_emit (menu, signals[OPEN], 0, location,
+		       ephy_gui_is_middle_click ());
+}
+
+static void
+open_bookmark_in_tab_cb (GtkAction *action,
+			 const char *location,
+			 gboolean dummy,
+			 EphyBookmarksMenu *menu)
+{
+	g_signal_emit (menu, signals[OPEN], 0, location, TRUE);
 }
 
 #define BMK_ACCEL_PATH_PREFIX "<Actions>/BmkActions/"
@@ -183,12 +216,14 @@ add_action_for_bookmark (EphyBookmarksMenu *menu,
 	g_snprintf (verb, sizeof (verb), BMK_VERB_FORMAT, id);
 	g_snprintf (apath, sizeof (apath), BMK_ACCEL_PATH_PREFIX "%s", verb);
 
-	action = ephy_bookmark_action_new (verb, id);
+	action = ephy_bookmark_action_new (verb, bmk);
 
 	gtk_action_set_accel_path (action, apath);
 
 	g_signal_connect (action, "open",
 			  G_CALLBACK (open_bookmark_cb), menu);
+	g_signal_connect (action, "open_in_tab",
+			  G_CALLBACK (open_bookmark_in_tab_cb), menu);
 
 	gtk_action_group_add_action (menu->priv->bmk_actions, action);
 	g_object_unref (action);
@@ -209,6 +244,9 @@ ensure_bookmark_actions (EphyBookmarksMenu *menu)
 	gtk_ui_manager_insert_action_group (menu->priv->manager,
 					    menu->priv->bmk_actions, -1);
 
+	g_signal_connect (menu->priv->bmk_actions, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
+
 	bookmarks = ephy_bookmarks_get_bookmarks (menu->priv->bookmarks);
 	children = ephy_node_get_children (bookmarks);
 	for (i = 0; i < children->len; i++)
@@ -354,10 +392,10 @@ create_submenu (EphyBookmarksMenu *menu,
 	if (folders == NULL || folders[0] == NULL)
 	{
 		g_strfreev (folders);
-		return g_strdup (BOOKMARKS_MENU_PATH);
+		return g_strdup (p->path);
 	}
 
-	path = g_string_new (BOOKMARKS_MENU_PATH);
+	path = g_string_new (p->path);
 	for (i = 0; folders[i] != NULL; i++)
 	{
 		folder = folders[i];
@@ -428,6 +466,9 @@ ephy_bookmarks_menu_rebuild (EphyBookmarksMenu *menu)
 	p->folder_actions = gtk_action_group_new ("FolderActions");
 	gtk_ui_manager_insert_action_group (p->manager, p->folder_actions, -1);
 
+	g_signal_connect (p->folder_actions, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
+
 	p->ui_id = gtk_ui_manager_new_merge_id (p->manager);
 
 	topics = ephy_bookmarks_get_keywords (p->bookmarks);
@@ -464,7 +505,7 @@ ephy_bookmarks_menu_rebuild (EphyBookmarksMenu *menu)
 
 	if (ephy_node_get_n_children (not_categorized) > 0)
 	{
-		create_menu (menu, not_categorized, BOOKMARKS_MENU_PATH);
+		create_menu (menu, not_categorized, p->path);
 	}
 
 	g_list_free (node_list);
@@ -518,27 +559,21 @@ ephy_bookmarks_menu_maybe_update (EphyBookmarksMenu *menu)
 	}
 }
 
-static void
-ephy_bookmarks_menu_set_window (EphyBookmarksMenu *menu,
-				EphyWindow *window)
-{
-	menu->priv->window = window;
-	menu->priv->manager = GTK_UI_MANAGER (window->ui_merge);
-}
-
 static void
 ephy_bookmarks_menu_set_property (GObject *object,
 				  guint prop_id,
 				  const GValue *value,
 				  GParamSpec *pspec)
 {
-        EphyBookmarksMenu *m = EPHY_BOOKMARKS_MENU (object);
+        EphyBookmarksMenu *menu = EPHY_BOOKMARKS_MENU (object);
 
         switch (prop_id)
         {
-                case PROP_WINDOW:
-                        ephy_bookmarks_menu_set_window
-				(m, g_value_get_object (value));
+		case PROP_PATH:
+			menu->priv->path = g_value_dup_string (value);
+			break;
+                case PROP_UI_MANAGER:
+			menu->priv->manager = g_value_get_object (value);
                         break;
         }
 }
@@ -549,14 +584,8 @@ ephy_bookmarks_menu_get_property (GObject *object,
                                   GValue *value,
                                   GParamSpec *pspec)
 {
-        EphyBookmarksMenu *m = EPHY_BOOKMARKS_MENU (object);
-
-        switch (prop_id)
-        {
-                case PROP_WINDOW:
-                        g_value_set_object (value, m->priv->window);
-                        break;
-        }
+	/* no readable properties */
+	g_return_if_reached ();
 }
 
 static void
@@ -616,6 +645,23 @@ bookmark_added_cb (EphyNode *bookmarks,
 
 	if (menu->priv->bmk_actions != NULL)
 	{
+		/* If the new bookmark has the node ID of one scheduled to
+		 * be removed, remove the old one first then add new one.
+		 * This works since the action name depends only on the
+		 * node ID. See bug #154805.
+		 */
+		GSList *l;
+
+		l = g_slist_find (menu->priv->removed_bmks,
+				 GUINT_TO_POINTER (ephy_node_get_id (bmk)));
+		if (l != NULL)
+		{
+			remove_action (l->data, menu->priv->bmk_actions);
+
+			menu->priv->removed_bmks = g_slist_delete_link
+				(menu->priv->removed_bmks, l);
+		}
+
 		add_action_for_bookmark (menu, bmk);
 
 		ephy_bookmarks_menu_maybe_update (menu);
@@ -673,7 +719,7 @@ ephy_bookmarks_menu_constructor (GType type,
 
 	menu = EPHY_BOOKMARKS_MENU (object);
 
-	g_assert (menu->priv->window != NULL);
+	g_assert (menu->priv->manager != NULL && menu->priv->path != NULL);
 
 	menu->priv->bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 	g_signal_connect_object (menu->priv->bookmarks, "tree_changed",
@@ -700,7 +746,7 @@ ephy_bookmarks_menu_constructor (GType type,
 					 G_OBJECT (menu));
 
 	action = gtk_ui_manager_get_action (menu->priv->manager,
-					    BOOKMARKS_MENU_PATH);
+					    menu->priv->path);
 	g_signal_connect_object (action, "activate",
 				 G_CALLBACK (activate_cb), menu, 0);
 
@@ -732,6 +778,8 @@ ephy_bookmarks_menu_finalize (GObject *o)
 		g_object_unref (p->folder_actions);
 	}
 
+	g_free (p->path);
+
 	LOG ("EphyBookmarksMenu finalised %p", o);
 
 	G_OBJECT_CLASS (parent_class)->finalize (o);
@@ -749,22 +797,45 @@ ephy_bookmarks_menu_class_init (EphyBookmarksMenuClass *klass)
 	object_class->set_property = ephy_bookmarks_menu_set_property;
 	object_class->get_property = ephy_bookmarks_menu_get_property;
 
+	signals[OPEN] =
+                g_signal_new ("open",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (EphyBookmarksMenuClass, open),
+                              NULL, NULL,
+                              ephy_marshal_VOID__STRING_BOOLEAN,
+                              G_TYPE_NONE,
+                              2,
+			      G_TYPE_STRING,
+			      G_TYPE_BOOLEAN);
+
 	g_object_class_install_property (object_class,
-                                         PROP_WINDOW,
-                                         g_param_spec_object ("window",
-                                                              "Window",
-                                                              "Parent window",
-                                                              EPHY_TYPE_WINDOW,
-                                                              G_PARAM_READWRITE |
+                                         PROP_PATH,
+                                         g_param_spec_string  ("path",
+                                                               "Path",
+                                                               "Merge path",
+                                                               NULL,
+                                                               G_PARAM_WRITABLE |
+							       G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+                                         PROP_UI_MANAGER,
+                                         g_param_spec_object ("ui-manager",
+                                                              "UI Manager",
+                                                              "UI Manager",
+                                                              GTK_TYPE_UI_MANAGER,
+                                                              G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof (EphyBookmarksMenuPrivate));
 }
 
 EphyBookmarksMenu *
-ephy_bookmarks_menu_new (EphyWindow *window)
+ephy_bookmarks_menu_new (GtkUIManager *manager,
+			 const char *path)
 {
 	return EPHY_BOOKMARKS_MENU (g_object_new (EPHY_TYPE_BOOKMARKS_MENU,
-						  "window", window,
+						  "ui-manager", manager,
+						  "path", path,
 						  NULL));
 }
