@@ -15,13 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-menu.h,v 1.5 2004/08/23 12:08:01 chpe Exp $
+ *  $Id: ephy-bookmarks-menu.h,v 1.6 2004/10/29 20:53:49 chpe Exp $
  */
 
 #ifndef EPHY_BOOKMARKS_MENU_H
 #define EPHY_BOOKMARKS_MENU_H
 
-#include "ephy-window.h"
+#include <glib-object.h>
+#include <gtk/gtkuimanager.h>
 
 G_BEGIN_DECLS
 
@@ -39,6 +40,10 @@ typedef struct _EphyBookmarksMenuPrivate	EphyBookmarksMenuPrivate;
 struct _EphyBookmarksMenuClass
 {
 	GObjectClass parent_class;
+
+	void (*open)          (EphyBookmarksMenu *menu,
+			       const char *address,
+			       gboolean open_in_new);
 };
 
 struct _EphyBookmarksMenu
@@ -51,7 +56,8 @@ struct _EphyBookmarksMenu
 
 GType              ephy_bookmarks_menu_get_type		(void);
 
-EphyBookmarksMenu *ephy_bookmarks_menu_new		(EphyWindow *window);
+EphyBookmarksMenu *ephy_bookmarks_menu_new		(GtkUIManager *manager,
+							 const char *path);
 
 G_END_DECLS
 
