@@ -16,12 +16,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-encoding-dialog.c,v 1.13 2004/01/23 18:07:39 chpe Exp $
+ *  $Id: ephy-encoding-dialog.c,v 1.19 2004/11/02 20:57:20 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-encoding-dialog.h"
 #include "ephy-encodings.h"
@@ -86,7 +84,7 @@ ephy_encoding_dialog_get_type (void)
 {
 	static GType type = 0;
 
-	if (type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -113,12 +111,12 @@ static void
 sync_embed_cb (EphyEncodingDialog *dialog, GParamSpec *pspec, gpointer dummy)
 {
 	EphyEmbed *embed;
-	EphyEncodingInfo *info;
 	EphyNode *node;
         GtkTreeSelection *selection;
         GtkTreeModel *model;
         GList *rows;
 	GtkWidget *button;
+	char *encoding;
 	gboolean is_automatic;
 
 	dialog->priv->update_tag = TRUE;
@@ -126,10 +124,10 @@ sync_embed_cb (EphyEncodingDialog *dialog, GParamSpec *pspec, gpointer dummy)
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG (dialog));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	info = ephy_embed_get_encoding_info (embed);
-	if (info == NULL) return;
+	encoding = ephy_embed_get_encoding (embed);
+	if (encoding == NULL) return;
 
-	node = ephy_encodings_get_node (dialog->priv->encodings, info->encoding, TRUE);
+	node = ephy_encodings_get_node (dialog->priv->encodings, encoding, TRUE);
 	g_assert (EPHY_IS_NODE (node));
 
 	/* select the current encoding in the list view */
@@ -153,12 +151,12 @@ sync_embed_cb (EphyEncodingDialog *dialog, GParamSpec *pspec, gpointer dummy)
 		g_list_free (rows);
 	}
 
-	is_automatic = ephy_encoding_info_is_automatic (info);
+	is_automatic = ephy_embed_has_automatic_encoding (embed);
 
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[AUTOMATIC_PROP].id);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), is_automatic);
 
-	ephy_encoding_info_free (info);
+	g_free (encoding);
 
 	dialog->priv->update_tag = FALSE;
 }
@@ -199,27 +197,18 @@ static void
 activate_choice (EphyEncodingDialog *dialog)
 {
 	EphyEmbed *embed;
-	EphyEncodingInfo *info;
 	GtkWidget *button;
 	gboolean is_automatic;
 
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG (dialog));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	info = ephy_embed_get_encoding_info (embed);
-	if (info == NULL) return;
-
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[AUTOMATIC_PROP].id);
 	is_automatic = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 
 	if (is_automatic)
 	{
-		/* only unset if it was forced before */
-		if ((info->forced_encoding != NULL && info->forced_encoding[0] != '\0')
-		    || info->encoding_source >= EMBED_ENCODING_PARENT_FORCED)
-		{
-			ephy_embed_set_encoding (embed, "");
-		}
+		ephy_embed_set_encoding (embed, "");
 	}
 	else if (dialog->priv->selected_node != NULL)
 	{
@@ -228,16 +217,10 @@ activate_choice (EphyEncodingDialog *dialog)
 		code = ephy_node_get_property_string (dialog->priv->selected_node,
 						      EPHY_NODE_ENCODING_PROP_ENCODING);
 
-		/* only force it if it's different from active */
-		if (info->encoding && strcmp (info->encoding, code) != 0)
-		{
-			ephy_embed_set_encoding (embed, code);
+		ephy_embed_set_encoding (embed, code);
 
-			ephy_encodings_add_recent (dialog->priv->encodings, code);
-		}
+		ephy_encodings_add_recent (dialog->priv->encodings, code);
 	}
-
-	ephy_encoding_info_free (info);
 }
 
 void
@@ -313,9 +296,6 @@ ephy_encoding_dialog_init (EphyEncodingDialog *dialog)
 		EPHY_ENCODINGS (ephy_embed_shell_get_encodings
 				(EPHY_EMBED_SHELL (ephy_shell)));
 
-	dialog->priv->update_tag = FALSE;
-	dialog->priv->selected_node = NULL;
-
 	ephy_dialog_construct (EPHY_DIALOG (dialog),
 			       properties,
 			       ephy_file ("epiphany.glade"),
@@ -330,11 +310,13 @@ ephy_encoding_dialog_init (EphyEncodingDialog *dialog)
 	ephy_node_view_add_column (EPHY_NODE_VIEW (treeview), _("Encodings"),
 				   G_TYPE_STRING,
 				   EPHY_NODE_ENCODING_PROP_TITLE_ELIDED,
-				   -1,
-				   EPHY_NODE_VIEW_AUTO_SORT |
 				   EPHY_NODE_VIEW_SEARCHABLE,
 				   NULL);
 
+	ephy_node_view_set_sort (EPHY_NODE_VIEW (treeview), G_TYPE_STRING,
+				 EPHY_NODE_ENCODING_PROP_TITLE_ELIDED,
+				 GTK_SORT_ASCENDING);
+
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(treeview), FALSE);
@@ -404,5 +386,7 @@ ephy_encoding_dialog_new (EphyWindow *parent)
 {
 	return g_object_new (EPHY_TYPE_ENCODING_DIALOG,
 			     "parent-window", parent,
+			     "default-width", 350,
+		             "default-height", 420,
 			     NULL);
 }
