@@ -17,12 +17,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-extensions-manager.c,v 1.14 2004/10/31 18:37:00 chpe Exp $
+ *  $Id: ephy-extensions-manager.c,v 1.18 2004/11/18 15:18:55 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-extensions-manager.h"
 
@@ -37,7 +35,6 @@
 
 #include <libxml/tree.h>
 #include <libxml/xmlreader.h>
-#include <libxml/xmlschemas.h>
 
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -46,6 +43,13 @@
 #include <dirent.h>
 #include <string.h>
 
+/* define to validate the extensions files */
+/* #define ENABLE_VALIDATION */
+
+#ifdef ENABLE_VALIDATION
+#include <libxml/xmlschemas.h>
+#endif
+
 #define CONF_LOADED_EXTENSIONS	"/apps/epiphany/general/active_extensions"
 #define SCHEMA_FILE		"/epiphany-extension.xsd"
 
@@ -62,8 +66,10 @@ struct _EphyExtensionsManagerPrivate
 	GList *windows;
 	guint active_extensions_notifier_id;
 
+#ifdef ENABLE_VALIDATION
 	xmlSchemaPtr schema;
 	xmlSchemaValidCtxtPtr schema_ctxt;
+#endif
 };
 
 typedef struct
@@ -385,7 +391,7 @@ ephy_extensions_manager_load_string (EphyExtensionsManager *manager,
 		g_warning ("Couldn't read '%s' data\n", identifier);
 		return;
 	}
-#if 0
+#ifdef ENABLE_VALIDATION
 	/* Validate the extension description */
 	if (manager->priv->schema_ctxt)
 	{
@@ -689,6 +695,7 @@ ephy_extensions_manager_load_file (EphyExtensionsManager *manager,
 	ephy_extensions_manager_load_string (manager, identifier, xml);
 
 	g_free (identifier);
+	g_free (xml);
 }
 
 static int
@@ -934,7 +941,7 @@ load_file_from_monitor (EphyExtensionsManager *manager,
 	 */
 	char *identifier;
 	char *xml;
-	int len;
+	gsize len;
 	GError *err = NULL;
 
 	g_file_get_contents (path, &xml, &len, &err);
@@ -947,7 +954,10 @@ load_file_from_monitor (EphyExtensionsManager *manager,
 		return;
 	}
 
-	if (len == 0) return;
+	if (len == 0) {
+		g_free (xml);
+		return;
+	}
 
 	identifier = path_to_identifier (path);
 	g_return_if_fail (identifier != NULL);
@@ -956,12 +966,14 @@ load_file_from_monitor (EphyExtensionsManager *manager,
 				(GCompareFunc) find_extension_info) != NULL)
 	{
 		g_free (identifier);
+		g_free (xml);
 		return;
 	}
 
 	ephy_extensions_manager_load_string (manager, identifier, xml);
 
 	g_free (identifier);
+	g_free (xml);
 
 	sync_loaded_extensions (manager);
 }
@@ -1056,6 +1068,7 @@ active_extensions_notifier (GConfClient *client,
 	sync_loaded_extensions (manager);
 }
 
+#ifdef ENABLE_VALIDATION
 static void
 xml_error_cb (EphyExtensionsManager *manager,
 	      const char *msg,
@@ -1112,6 +1125,7 @@ init_schema_ctxt (EphyExtensionsManager *manager)
 				 (xmlSchemaValidityWarningFunc) xml_error_cb,
 				 manager);
 }
+#endif
 
 static void
 ephy_extensions_manager_init (EphyExtensionsManager *manager)
@@ -1130,7 +1144,9 @@ ephy_extensions_manager_startup (EphyExtensionsManager *manager)
 
 	LOG ("EphyExtensionsManager startup")
 
+#ifdef ENABLE_VALIDATION
 	init_schema_ctxt (manager);
+#endif
 
 	/* load the extensions descriptions */
 	path = g_build_filename (ephy_dot_dir (), "extensions", NULL);
@@ -1171,6 +1187,7 @@ ephy_extensions_manager_finalize (GObject *object)
 
 	g_list_free (priv->windows);
 
+#ifdef ENABLE_VALIDATION
 	if (priv->schema)
 	{
 		xmlSchemaFree (priv->schema);
@@ -1179,6 +1196,7 @@ ephy_extensions_manager_finalize (GObject *object)
 	{
 		xmlSchemaFreeValidCtxt (priv->schema_ctxt);
 	}
+#endif
 
 	parent_class->finalize (object);
 }
