@@ -17,12 +17,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-notebook.c,v 1.54 2004/10/27 19:54:57 chpe Exp $
+ *  $Id: ephy-notebook.c,v 1.59 2004/11/27 23:41:02 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-notebook.h"
 #include "ephy-stock-icons.h"
@@ -193,27 +191,6 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 	g_type_class_add_private (object_class, sizeof(EphyNotebookPrivate));
 }
 
-static gboolean
-is_in_notebook_window (EphyNotebook *notebook,
-		       gint abs_x, gint abs_y)
-{
-	gint x, y;
-	gint rel_x, rel_y;
-	gint width, height;
-	GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET(notebook));
-	GdkWindow *window = GTK_WIDGET(toplevel)->window;
-
-	gdk_window_get_origin (window, &x, &y);
-	rel_x = abs_x - x;
-	rel_y = abs_y - y;
-
-	x = GTK_WIDGET(notebook)->allocation.x;
-	y = GTK_WIDGET(notebook)->allocation.y;
-	height = GTK_WIDGET(notebook)->allocation.height;
-	width  = GTK_WIDGET(notebook)->allocation.width;
-	return ((rel_x>=x) && (rel_y>=y) && (rel_x<=x+width) && (rel_y<=y+height));
-}
-
 static EphyNotebook *
 find_notebook_at_pointer (gint abs_x, gint abs_y)
 {
@@ -221,6 +198,7 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 	gpointer toplevel = NULL;
 	gint x, y;
 
+	/* FIXME multi-head */
 	win_at_pointer = gdk_window_at_pointer (&x, &y);
 	if (win_at_pointer == NULL)
 	{
@@ -236,20 +214,24 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 	/* toplevel should be an EphyWindow */
 	if (toplevel != NULL && EPHY_IS_WINDOW (toplevel))
 	{
-		EphyNotebook *notebook;
-
-		notebook = EPHY_NOTEBOOK (ephy_window_get_notebook
-					    (EPHY_WINDOW (toplevel)));
-
-		if (is_in_notebook_window (notebook, abs_x, abs_y))
-		{
-			return notebook;
-		}
+		return EPHY_NOTEBOOK (ephy_window_get_notebook
+					(EPHY_WINDOW (toplevel)));
 	}
 
 	return NULL;
 }
 
+static gboolean
+is_in_notebook_window (EphyNotebook *notebook,
+		       gint abs_x, gint abs_y)
+{
+	EphyNotebook *nb_at_pointer;
+
+	nb_at_pointer = find_notebook_at_pointer (abs_x, abs_y);
+
+	return nb_at_pointer == notebook;
+}
+
 static gint
 find_tab_num_at_pos (EphyNotebook *notebook, gint abs_x, gint abs_y)
 {
@@ -841,12 +823,10 @@ tab_label_style_set_cb (GtkWidget *label,
 static GtkWidget *
 build_tab_label (EphyNotebook *nb, EphyTab *tab)
 {
-	GtkWidget *window, *hbox, *label_hbox, *label_ebox;
+	GtkWidget *hbox, *label_hbox, *label_ebox;
 	GtkWidget *label, *close_button, *image, *spinner, *icon;
 	GtkIconSize close_icon_size;
 
-	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
-
 	/* set hbox spacing and label padding (see below) so that there's an
 	 * equal amount of space around the label */
 	hbox = gtk_hbox_new (FALSE, 4);
@@ -956,15 +936,15 @@ ephy_notebook_add_tab (EphyNotebook *nb,
 
 	g_signal_emit (G_OBJECT (nb), signals[TAB_ADDED], 0, tab);
 
+	/* The signal handler may have reordered the tabs */
+	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
+
 	if (jump_to)
 	{
-		/* The signal handler may have reordered the tabs */
-		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET(tab));
-
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
-					       position);
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), position);
 		g_object_set_data (G_OBJECT (tab), "jump_to",
 				   GINT_TO_POINTER (jump_to));
+
 	}
 }
 
