@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2003  David Bordoley
+ *  Copyright (C) 2003-2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,12 +16,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tabs-menu.c,v 1.25 2004/04/13 20:39:28 chpe Exp $
+ *  $Id: ephy-tabs-menu.c,v 1.30 2004/11/02 20:57:21 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-tabs-menu.h"
 #include "ephy-string.h"
@@ -28,6 +27,8 @@
 #include "ephy-shell.h"
 #include "ephy-debug.h"
 
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmenuitem.h>
 #include <gtk/gtkaccelmap.h>
 #include <gtk/gtkaction.h>
 #include <gtk/gtkradioaction.h>
@@ -59,9 +60,9 @@ enum
 GType
 ephy_tabs_menu_get_type (void)
 {
-        static GType ephy_tabs_menu_type = 0;
+        static GType type = 0;
 
-        if (ephy_tabs_menu_type == 0)
+        if (G_UNLIKELY (type == 0))
         {
                 static const GTypeInfo our_info =
                 {
@@ -76,12 +77,12 @@ ephy_tabs_menu_get_type (void)
                         (GInstanceInitFunc) ephy_tabs_menu_init
                 };
 
-                ephy_tabs_menu_type = g_type_register_static (G_TYPE_OBJECT,
-							      "EphyTabsMenu",
-							      &our_info, 0);
+                type = g_type_register_static (G_TYPE_OBJECT,
+					       "EphyTabsMenu",
+					       &our_info, 0);
         }
 
-        return ephy_tabs_menu_type;
+        return type;
 }
 
 static void
@@ -122,19 +123,35 @@ tabs_reordered_cb (EphyNotebook *notebook, EphyTabsMenu *menu)
 	ephy_tabs_menu_update (menu);
 }
 
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+		  GtkAction *action,
+		  GtkWidget *proxy,
+		  gpointer dummy)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		gtk_label_set_ellipsize (GTK_LABEL (GTK_BIN (proxy)->child),
+					 PANGO_ELLIPSIZE_END);
+	}
+}
+
 static void
 ephy_tabs_menu_set_window (EphyTabsMenu *menu, EphyWindow *window)
 {
 	GtkWidget *notebook;
-	GtkUIManager *merge;
+	GtkUIManager *manager;
 
 	menu->priv->window = window;
 
-	merge = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 	menu->priv->action_group = gtk_action_group_new ("TabsActions");
-	gtk_ui_manager_insert_action_group (merge, menu->priv->action_group, 0);
+	gtk_ui_manager_insert_action_group (manager, menu->priv->action_group, -1);
 	g_object_unref (menu->priv->action_group);
 
+	g_signal_connect (menu->priv->action_group, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
+
 	notebook = ephy_window_get_notebook (window);
 	g_signal_connect_object (notebook, "tab_added",
 			         G_CALLBACK (tab_added_cb), menu, 0);
@@ -187,8 +204,8 @@ ephy_tabs_menu_class_init (EphyTabsMenuClass *klass)
 
 	g_object_class_install_property (object_class,
                                          PROP_WINDOW,
-                                         g_param_spec_object ("EphyWindow",
-                                                              "EphyWindow",
+                                         g_param_spec_object ("window",
+                                                              "Window",
                                                               "Parent window",
                                                               EPHY_TYPE_WINDOW,
                                                               G_PARAM_READWRITE |
@@ -201,21 +218,18 @@ static void
 ephy_tabs_menu_init (EphyTabsMenu *menu)
 {
 	menu->priv = EPHY_TABS_MENU_GET_PRIVATE (menu);
-
-	menu->priv->ui_id = 0;
-	menu->priv->action_group = NULL;
 }
 
 static void
 ephy_tabs_menu_clean (EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *p = menu->priv;
-	GtkUIManager *merge = GTK_UI_MANAGER (p->window->ui_merge);
+	GtkUIManager *manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (p->window));
 
 	if (p->ui_id > 0)
 	{
-		gtk_ui_manager_remove_ui (merge, p->ui_id);
-		gtk_ui_manager_ensure_update (merge);
+		gtk_ui_manager_remove_ui (manager, p->ui_id);
+		gtk_ui_manager_ensure_update (manager);
 		p->ui_id = 0;
 	}
 }
@@ -224,7 +238,7 @@ EphyTabsMenu *
 ephy_tabs_menu_new (EphyWindow *window)
 {
 	return EPHY_TABS_MENU (g_object_new (EPHY_TYPE_TABS_MENU,
-					     "EphyWindow", window,
+					     "window", window,
 					     NULL));
 }
 
@@ -275,17 +289,13 @@ tab_set_action_accelerator (GtkActionGroup *action_group,
 void
 ephy_tabs_menu_update (EphyTabsMenu *menu)
 {
-	EphyTabsMenuPrivate *p;
-	GtkUIManager *merge;
+	EphyTabsMenuPrivate *p = menu->priv;
+	GtkUIManager *manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (p->window));
 	EphyTab *tab;
 	GtkAction *action;
 	guint i = 0;
 	GList *tabs = NULL, *l;
 
-	g_return_if_fail (EPHY_IS_TABS_MENU (menu));
-	p = menu->priv;
-	merge = GTK_UI_MANAGER (p->window->ui_merge);
-	
 	LOG ("Rebuilding open tabs menu")
 
 	START_PROFILER ("Rebuilding tabs menu")
@@ -296,7 +306,7 @@ ephy_tabs_menu_update (EphyTabsMenu *menu)
 
 	if (g_list_length (tabs) == 0) return;
 
-	p->ui_id = gtk_ui_manager_new_merge_id (merge);
+	p->ui_id = gtk_ui_manager_new_merge_id (manager);
 
 	for (l = tabs; l != NULL; l = l->next)
 	{
@@ -310,7 +320,7 @@ ephy_tabs_menu_update (EphyTabsMenu *menu)
 
 		tab_set_action_accelerator (p->action_group, action, i++);
 
-		gtk_ui_manager_add_ui (merge, p->ui_id,
+		gtk_ui_manager_add_ui (manager, p->ui_id,
 				       "/menubar/TabsMenu/TabsOpen",
 				       name, action_name,
 				       GTK_UI_MANAGER_MENUITEM, FALSE);
