@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: popup-commands.c,v 1.44 2004/09/26 15:51:17 chpe Exp $
+ *  $Id: popup-commands.c,v 1.45 2004/11/02 20:57:21 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "popup-commands.h"
 #include "ephy-shell.h"
