@@ -17,12 +17,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: toolbar.c,v 1.81.4.1 2004/09/13 10:32:13 chpe Exp $
+ *  $Id: toolbar.c,v 1.89 2004/11/27 21:53:23 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "toolbar.h"
 #include "ephy-favicon-action.h"
@@ -63,6 +61,14 @@ enum
 	PROP_WINDOW
 };
 
+enum
+{
+	ACTIVATION_FINISHED,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static GObjectClass *parent_class = NULL;
 
 #define CONF_LOCKDOWN_DISABLE_ARBITRARY_URL  "/apps/epiphany/lockdown/disable_arbitrary_url"
@@ -76,14 +82,15 @@ struct ToolbarPrivate
 	gboolean updating_address;
 	GtkWidget *spinner;
 	guint disable_arbitrary_url_notifier_id;
+	gulong set_focus_handler;
 };
 
 GType
 toolbar_get_type (void)
 {
-        static GType toolbar_type = 0;
+        static GType type = 0;
 
-        if (toolbar_type == 0)
+        if (G_UNLIKELY (type == 0))
         {
                 static const GTypeInfo our_info =
                 {
@@ -98,13 +105,12 @@ toolbar_get_type (void)
                         (GInstanceInitFunc) toolbar_init
                 };
 
-                toolbar_type = g_type_register_static (EGG_TYPE_EDITABLE_TOOLBAR,
-						       "Toolbar",
-						       &our_info, 0);
+                type = g_type_register_static (EGG_TYPE_EDITABLE_TOOLBAR,
+					       "Toolbar",
+					       &our_info, 0);
         }
 
-        return toolbar_type;
-
+        return type;
 }
 
 static void
@@ -251,6 +257,16 @@ toolbar_class_init (ToolbarClass *klass)
 	widget_class->realize = toolbar_realize;
 	widget_class->unrealize = toolbar_unrealize;
 
+	signals[ACTIVATION_FINISHED] =
+                g_signal_new ("activation-finished",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (ToolbarClass, activation_finished),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
 	g_object_class_install_property (object_class,
                                          PROP_WINDOW,
                                          g_param_spec_object ("window",
@@ -295,6 +311,10 @@ toolbar_setup_actions (Toolbar *t)
 			       "label", _("Back"),
 			       "stock_id", GTK_STOCK_GO_BACK,
 			       "tooltip", _("Go back"),
+			       /* this is the tooltip on the Back button's drop-down arrow, which will show
+			        * a menu with all sites you can go 'back' to
+				*/
+			       "arrow-tooltip", _("Back history"),
 			       "window", t->priv->window,
 			       "direction", EPHY_NAVIGATION_DIRECTION_BACK,
 			       "is_important", TRUE,
@@ -309,6 +329,10 @@ toolbar_setup_actions (Toolbar *t)
 			       "label", _("Forward"),
 			       "stock_id", GTK_STOCK_GO_FORWARD,
 			       "tooltip", _("Go forward"),
+			       /* this is the tooltip on the Forward button's drop-down arrow, which will show
+				* a menu with all sites you can go 'forward' to
+				*/
+			       "arrow-tooltip", _("Forward history"),
 			       "window", t->priv->window,
 			       "direction", EPHY_NAVIGATION_DIRECTION_FORWARD,
 			       NULL);
@@ -321,7 +345,11 @@ toolbar_setup_actions (Toolbar *t)
 			       "name", "NavigationUp",
 			       "label", _("Up"),
 			       "stock_id", GTK_STOCK_GO_UP,
-			       "tooltip", _("Go up"),
+			       "tooltip", _("Go up one level"),
+			       /* this is the tooltip on the Up button's drop-down arrow, which will show
+				* a menu with al sites you can go 'up' to
+				*/
+			       "arrow-tooltip", _("List of upper levels"),
 			       "window", t->priv->window,
 			       "direction", EPHY_NAVIGATION_DIRECTION_UP,
 			       NULL);
@@ -338,6 +366,7 @@ toolbar_setup_actions (Toolbar *t)
 			       "label", _("Address Entry"),
 			       "stock_id", EPHY_STOCK_ENTRY,
 			       "tooltip", _("Enter a web address to open, or a phrase to search for on the web"),
+			       "visible-overflown", FALSE,
 			       NULL);
 	g_signal_connect (action, "go_location",
 			  G_CALLBACK (go_location_cb), t->priv->window);
@@ -362,7 +391,9 @@ toolbar_setup_actions (Toolbar *t)
 	action = g_object_new (EPHY_TYPE_FAVICON_ACTION,
 			       "name", "Favicon",
 			       "label", _("Favicon"),
+			       "tooltip", _("Drag and drop this icon to create a link to this page"),
 			       "window", t->priv->window,
+			       "visible-overflown", FALSE,
 			       NULL);
 	gtk_action_group_add_action (t->priv->action_group, action);
 	g_object_unref (action);
@@ -387,7 +418,7 @@ toolbar_set_window (Toolbar *t, EphyWindow *window)
 	g_return_if_fail (t->priv->window == NULL);
 
 	t->priv->window = window;
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	toolbar_setup_actions (t);
 	gtk_ui_manager_insert_action_group (manager,
@@ -449,6 +480,12 @@ toolbar_finalize (GObject *object)
 	Toolbar *t = EPHY_TOOLBAR (object);
 	EggEditableToolbar *eggtoolbar = EGG_EDITABLE_TOOLBAR (object);
 
+	if (t->priv->set_focus_handler != 0)
+	{
+		g_signal_handler_disconnect (t->priv->window,
+					     t->priv->set_focus_handler);
+	}
+
 	eel_gconf_notification_remove
 		(t->priv->disable_arbitrary_url_notifier_id);
 
@@ -465,26 +502,70 @@ toolbar_finalize (GObject *object)
 	LOG ("Toolbar finalized")
 }
 
-Toolbar *
-toolbar_new (EphyWindow *window)
+static void 
+maybe_finish_activation_cb (EphyWindow *window,
+			    GtkWidget *widget,
+			    Toolbar *toolbar)
 {
-	return EPHY_TOOLBAR (g_object_new (EPHY_TYPE_TOOLBAR,
-					   "window", window,
-					   "ui-manager", window->ui_merge,
-					   NULL));
+	GtkWidget *wtoolbar = GTK_WIDGET (toolbar);
+
+	while (widget != NULL && widget != wtoolbar)
+	{
+		widget = widget->parent;
+	}
+
+	/* if widget == toolbar, the new focus widget is in the toolbar, so we
+	 * don't deactivate.
+	 */
+	if (widget != wtoolbar)
+	{
+		g_signal_handler_disconnect (window,
+					     toolbar->priv->set_focus_handler);
+		toolbar->priv->set_focus_handler = 0;
+
+		g_signal_emit (toolbar, signals[ACTIVATION_FINISHED], 0);
+	}
 }
 
 void
-toolbar_activate_location (Toolbar *t)
+toolbar_activate_location (Toolbar *toolbar)
 {
 	GtkActionGroup *action_group;
 	GtkAction *action;
+	GSList *proxies;
+	GtkWidget *entry = NULL;
+	gboolean visible;
 
-	action_group = t->priv->action_group;
+	action_group = toolbar->priv->action_group;
 	action = gtk_action_group_get_action (action_group, "Location");
 
-	gtk_action_activate (action);
-}
+	proxies = gtk_action_get_proxies (action);
+
+	if (proxies != NULL && EPHY_IS_LOCATION_ENTRY (proxies->data))
+	{
+		entry = GTK_WIDGET (proxies->data);
+	}
+
+	if (entry == NULL)
+	{
+		/* happens when the user has removed the location entry from
+		 * the toolbars.
+		 */
+		return;
+	}
+
+	ephy_location_entry_activate (EPHY_LOCATION_ENTRY (entry));
+
+	g_object_get (G_OBJECT (toolbar), "visible", &visible, NULL);
+	if (visible == FALSE)
+	{
+		gtk_widget_show (GTK_WIDGET (toolbar));
+		toolbar->priv->set_focus_handler =
+			g_signal_connect (toolbar->priv->window, "set-focus",
+					  G_CALLBACK (maybe_finish_activation_cb),
+					  toolbar);
+	}
+}	
 
 void
 toolbar_spinner_start (Toolbar *t)
@@ -567,3 +648,12 @@ toolbar_update_zoom (Toolbar *t, float zoom)
 	action = gtk_action_group_get_action (action_group, "Zoom");
 	g_object_set (action, "zoom", zoom, NULL);
 }
+
+Toolbar *
+toolbar_new (EphyWindow *window)
+{
+	return EPHY_TOOLBAR (g_object_new (EPHY_TYPE_TOOLBAR,
+					   "window", window,
+					   "ui-manager", ephy_window_get_ui_manager (window),
+					   NULL));
+}
