@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.131.4.3 2004/10/08 12:16:39 marco Exp $
+ *  $Id: window-commands.c,v 1.144 2004/11/25 19:01:26 marco Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
@@ -58,7 +56,7 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkicontheme.h>
 #include <gtk/gtktoggleaction.h>
-#include <libgnomeui/gnome-about.h>
+#include <gtk/gtkaboutdialog.h>
 #include <glib/gi18n.h>
 
 enum
@@ -417,7 +415,7 @@ window_cmd_file_save_as (GtkAction *action,
 	g_return_if_fail (embed != NULL);
 
 	persist = EPHY_EMBED_PERSIST
-		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_fc_title (persist, _("Save As"));
@@ -646,19 +644,6 @@ window_cmd_view_zoom_normal (GtkAction *action,
 	ephy_window_set_zoom (window, 1.0);
 }
 
-static GnomeVFSMimeApplication *
-get_editor_application (void)
-{
-	GnomeVFSMimeApplication *app;
-
-	app = gnome_vfs_mime_get_default_application ("text/plain");
-	if (app == NULL)
-	{
-		g_warning ("Cannot find a text editor.");
-	}
-	return app;
-}
-
 static void
 editor_open_uri (const char *address)
 {
@@ -670,12 +655,18 @@ editor_open_uri (const char *address)
 
 	uris = g_list_append (uris, canonical);
 
-	app = get_editor_application ();
+	app = gnome_vfs_mime_get_default_application ("text/plain");
 	if (app)
 	{
 		gnome_vfs_mime_application_launch (app, uris);
 		gnome_vfs_mime_application_free (app);
 	}
+	else
+	{
+		/* FIXME We should really warn the user here */
+
+		g_warning ("Cannot find a text editor.");
+	}
 
 	g_free (canonical);
 	g_list_free (uris);
@@ -694,48 +685,6 @@ save_source_completed_cb (EphyEmbedPersist *persist)
 	editor_open_uri (dest);
 }
 
-static gboolean
-editor_can_open_uri (char *address)
-{
-	GnomeVFSMimeApplication *app;
-	GnomeVFSURI *uri;
-	const char *scheme;
-	gboolean result = FALSE;
-
-	app = get_editor_application ();
-
-	uri = gnome_vfs_uri_new (address);
-	scheme = uri ? gnome_vfs_uri_get_scheme (uri) : NULL;
-
-	/* Open directly only read/write protocols, otherwise
-	   you just get extra network overhead without any advantage */
-	if (scheme && strcmp (scheme, "file") != 0)
-	{
-		scheme = NULL;
-	}
-	
-	if (scheme && app && app->supported_uri_schemes)
-	{
-		if (g_list_find_custom (app->supported_uri_schemes,
-                                        scheme, (GCompareFunc) strcmp))
-		{
-			result = TRUE;
-		}
-	}
-
-	if (uri)
-	{
-		gnome_vfs_uri_unref (uri);
-	}
-
-	if (app)
-	{
-		gnome_vfs_mime_application_free (app);
-	}
-
-	return result;
-}
-
 static void
 save_temp_source (EphyEmbed *embed)
 {
@@ -758,7 +707,7 @@ save_temp_source (EphyEmbed *embed)
 	}
 
 	persist = EPHY_EMBED_PERSIST
-		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_COPY_PAGE |
@@ -780,13 +729,15 @@ window_cmd_view_page_source (GtkAction *action,
 {
 	EphyEmbed *embed;
 	char *address;
+	char *scheme;
 
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
 	address = ephy_embed_get_location (embed, TRUE);
+	scheme = gnome_vfs_get_uri_scheme (address);
 
-	if (editor_can_open_uri (address))
+	if (strcmp (scheme, "file") == 0)
 	{
 		editor_open_uri (address);
 	}
@@ -795,6 +746,7 @@ window_cmd_view_page_source (GtkAction *action,
 		save_temp_source (embed);
 	}
 
+	g_free (scheme);
 	g_free (address);
 }
 
@@ -881,10 +833,9 @@ window_cmd_edit_toolbar (GtkAction *action,
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Toolbar Editor"));
         gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
 
-	ephy_gui_set_default_window_icon (GTK_WINDOW (dialog));
-
-	editor = egg_toolbar_editor_new (GTK_UI_MANAGER (window->ui_merge), model);
+	editor = egg_toolbar_editor_new (GTK_UI_MANAGER (ephy_window_get_ui_manager (window)), model);
 	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
 					 ephy_file ("epiphany-toolbar.xml"));
 	gtk_container_set_border_width (GTK_CONTAINER (EGG_TOOLBAR_EDITOR (editor)), 5);
@@ -930,10 +881,6 @@ window_cmd_help_about (GtkAction *action,
 {
 	static GtkWidget *about = NULL;
 	GtkWidget** ptr;
-	const char *icon_path;
-	GdkPixbuf *icon = NULL;
-	GtkIconTheme *icon_theme;
-	GtkIconInfo *icon_info;
 
 	static char *authors[] = {
 		"Marco Pesenti Gritti <marco@gnome.org>",
@@ -950,58 +897,35 @@ window_cmd_help_about (GtkAction *action,
 		NULL
 	};
 
-	/* Translator credits */
-	char *translator_credits = _("translator-credits");
-
 	if (about != NULL)
 	{
-		gdk_window_show(about->window);
-		gdk_window_raise(about->window);
+		gtk_window_present (GTK_WINDOW (about));
+
 		return;
 	}
 
-	/* FIXME: use the icon theme for the correct screen, not for the default screen */
-	icon_theme = gtk_icon_theme_get_default ();
-	icon_info = gtk_icon_theme_lookup_icon (icon_theme, "web-browser", -1, 0);
 
-	if (icon_info != NULL)
-	{
-		icon_path = gtk_icon_info_get_filename (icon_info);
+	about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
+			      "name", _("Epiphany"),
+			      "version", VERSION,
+			      "copyright", "Copyright \xc2\xa9 2002-2004 Marco Pesenti Gritti",
+			      "authors", authors,
+			      "documenters", documenters,
+			      "translator-credits", _("translator-credits"),
+			      "logo-icon-name", "web-browser",
+			      NULL);
 
-		if (icon_path != NULL)
-		{
-			icon = gdk_pixbuf_new_from_file (icon_path, NULL);
-		}
+	gtk_window_set_icon_name (GTK_WINDOW (about), "web-browser");
 
-		gtk_icon_info_free (icon_info);
-	}
-	else
-	{
-		g_warning ("Web browser gnome icon not found");
-	}
+	g_signal_connect (about, "response", G_CALLBACK (gtk_widget_destroy), NULL);
 
-	about = gnome_about_new(
-		       "Epiphany", VERSION,
-		       "Copyright \xc2\xa9 2002-2004 Marco Pesenti Gritti",
-		       _("A GNOME browser based on Mozilla"),
-		       (const char **)authors,
-		       (const char **)documenters,
-		       strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-		       icon);
-
-	gtk_window_set_icon (GTK_WINDOW (about), icon);
-
-	if (icon != NULL)
-	{
-		g_object_unref (icon);
-	}
-
-	gtk_window_set_transient_for (GTK_WINDOW (about),
-				      GTK_WINDOW (window));
+	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (window));
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);
 
 	ptr = &about;
 	g_object_add_weak_pointer (G_OBJECT (about), (gpointer *)ptr);
-	gtk_widget_show (about);
+
+	gtk_window_present (GTK_WINDOW (about));
 }
 
 void
